<?php

namespace db
{
	class users_advent extends \_tbl_line
	{
		protected static $table = 'users_advent';
		
		public static function is_completed($user_id, $type)
		{
			if (self::get("{$type}_completed", 'where id_user = ?', [$user_id]) > 0) {
				return true;
			} return false;
		}
		
		public static function boss_get($user_id, $type)
		{
			return self::get("$type", 'where id_user = ?', [$user_id]);
		}
		
		public static function boss_set($user_id, $type, $boss_id)
		{
			return self::set("$type = ?", 'where id_user = ?', [$boss_id, $user_id]);
		}
		
		public static function boss_timestamp_get($user_id, $type)
		{
			return self::get("{$type}_time", 'where id_user = ?', [$user_id]);
		}
		
		public static function boss_timestamp_set($user_id, $type)
		{
			return self::set("{$type}_time = ?", 'where id_user = ?', [time(), $user_id]);
		}
		
		public static function completed_set($user_id, $type, $value = 1)
		{
			return self::set("{$type}_completed = $value", 'where id_user = ?', [$user_id]);
		}
		
		public static function mutagen_collected_get($user_id)
		{
			return self::get('mutagen_collected', 'where id_user = ?', [$user_id]);
		}
		
		public static function mutagen_collected_timestamp_get($user_id)
		{
			return self::get('mutagen_collected_timestamp', 'where id_user = ?', [$user_id]);
		}
		
		public static function mutagen_collected_set($num, $user_id)
		{
			return self::set('mutagen_collected = ?', 'where id_user = ?', [$num, $user_id]);
		}
		
		public static function mutagen_collected_increase($num, $user_id)
		{
			return self::set('mutagen_collected = mutagen_collected + ?', 'where id_user = ?', [$num, $user_id]);
		}
		
		public static function mutagen_collected_timestamp_set($time, $user_id)
		{
			if ($time === false) {
				$time = time();
			}
			return self::set('mutagen_collected_timestamp = ?', 'where id_user = ?', [$time, $user_id]);
		}
	}
}

namespace advent
{
	class enemy
	{
		const RECOVERY_SECONDS = 3600;
		
		public
			$user_id,
			$type;
		
		public function __construct()
		{
			if ( ! $this->user_id) {
				$this->user_id = ID;
			}
		}
		
		/* метод создаёт врага если приключение пройдено */
		public function create()
		{
			if ($this->exists()) {
				return false;
			}
			$enemy_id = $this->random_id();
			\db\users_advent::boss_set($this->user_id, $this->type, $enemy_id);
		}
		
		public function recovery_seconds_left()
		{
			return (\db\users_advent::boss_timestamp_get($this->user_id, $this->type) + self::RECOVERY_SECONDS) - time();
		}
		
		public function exists()
		{
			if (is_null(\db\users_advent::boss_get($this->user_id, $this->type))) {
				return false;
			} return true;
		}
		
		private function random_id()
		{
			$sth = \db::$pdo->query("select count(*) from _advent_enemies where type = '$this->type' limit 1");
			$enemies_num = $sth->fetchColumn();
			
			$start = mt_rand(0, ($enemies_num - 1));
			$sth = \db::$pdo->query("select id from _advent_enemies where type = '$this->type' limit $start, 1");
			return $sth->fetchColumn();
		}
	}
	
	class battle
	{
		public
			$user_id,
			$type;
		
		public function __construct()
		{
			if ( ! $this->user_id) {
				$this->user_id = ID;
			}
		}

		/* свободный бой (когда приключение пройдено) */
		public function free()
		{
			$enemy = new enemy;
			$enemy->user_id = $this->user_id;
			$enemy->type = $this->type;
			
			if ($enemy->recovery_seconds_left() > 0) {
				return false;
			}
			
			$expr_reward = mt_rand(4000, 5000);
			$bucks_reward = mt_rand(10000, 12000);
			
			$expr = new \expr;
			$expr->base = $expr_reward;
			$expr->add();
			
			$currency = new \currency;
			$currency->base = $bucks_reward;
			$currency->bucks_add();
			
			// запись времени для отката
			\db\users_advent::boss_timestamp_set($this->user_id, $this->type);
			
			// удалить босса
			\db\users_advent::boss_set($this->user_id, $this->type, null);
			
			// собрать мутаген
			\advent_mutagen::gather($this->user_id);
			
			$user_statistic = new \user_statistic($this->user_id);
			$user_statistic->increase_statistic('advent', 1);
		}
		
		/* бой в рамках приключения */
		public function advent()
		{
			$enemy = new enemy;
			$enemy->user_id = $this->user_id;
			$enemy->type = $this->type;
			
			if ($enemy->recovery_seconds_left() > 0) {
				return false;
			}
			
			$type_id = \db\users_advent::boss_get($this->user_id, $this->type);
			$boss_id = \advent::get_id_boss_by_type_a_id_type($type_id, $this->type);
			$need_param = \advent::get_need_param($boss_id); // нужный параметр
			$user_param = \uc::get_param($this->user_id, $this->type); // параметр игрока
			
			$complex = \advent::boss_complexity_by_stat($user_param, $need_param); // сложность
			
			$exp_reward = \advent::get_exp_reward_by_id_boss($boss_id); // какая награда за этого босса опыт
			$bucks_reward = \advent::get_bucks_reward_by_id_boss($boss_id); // какая награда за этого босса в баксах
			
			$vict = \advent::victory_or_lost($this->user_id, $complex);
			if ($vict) { // победа
				if ($boss_id == \advent::get_last_id_boss_by_type($this->type)) { // если это последний босс категории
					\advent::etap_update($this->user_id, $this->type, null); // установить null в поле определенного типа (значит что больше боссов нет)
					\db\users_advent::completed_set($this->user_id, $this->type);
					/* v($this->user_id);
					v($this->type);
					die; */
				}
				else {
					\advent::etap_update($this->user_id, $this->type); // перейти на следующий этап
				}
				\advent::set_result($this->user_id, 1); // установить победу (users_dyn)
				$mutagen_reward = \advent::get_mutagen_reward_by_id_boss($boss_id); // какая награда мутагена за босса
				// начисление награды игроку
				\set_mutagen($mutagen_reward, $this->user_id);
				\user_cards::card_up_ctrl(\advent::get_id_card($boss_id), 1, $this->user_id); // добавить картку
			}
			else { // поражение
				\advent::set_result($this->user_id, 0); // установить поражение (users_dyn)
				$exp_reward = round(\percentage($exp_reward, 15)); // за поражение дается только малая часть награды
				$bucks_reward = round(\percentage($bucks_reward, 10)); // за поражение дается только малая часть награды
			}
			\user_exp::exp_set($exp_reward, $this->user_id);
			\set_bucks($bucks_reward, $this->user_id);
			\advent::boss_type_time_update($this->user_id, $this->type); // запись времени (для отката)
			\advent_mutagen::gather($this->user_id);
			
			$user_statistic = new \user_statistic($this->user_id);
			$user_statistic->increase_statistic('advent', 1);
		}
	}
}

namespace
{
	class advent
	{
		const TIME_RECOVERY = 3600; // время восстановления приключения
		
		// узнать этап (id_type в _advent_enemies) определонного типа босса у игрока
		// тип health, power, agil, intell
		public static function get_user_etap_boss_type($id_user, $type)
		{
			if ($type != 'health' && $type != 'power' && $type != 'agil' && $type != 'intell') {
				return false;
			}
			$sql = "select $type from users_advent where id_user = ? limit 1";
			$sth = db::$pdo->prepare($sql);
			$sth->execute([$id_user]);
			return $sth->fetch(PDO::FETCH_NUM)[0];
		}
		
		// получить массив со всеми этапами игрока
		public static function get_user_etap_boss_type_all($id_user)
		{
			$sql = "select health, power, agil, intell from users_advent where id_user = ? limit 1";
			$sth = db::$pdo->prepare($sql);
			$sth->execute([$id_user]);
			return $sth->fetch(PDO::FETCH_ASSOC);
		}
		
		// узнать имя босса по id
		public static function get_boss_name_by_id($id_boss)
		{
			return cards::card_name_get(self::get_id_card($id_boss));
		}
		
		// узнать id-карточки
		public static function get_id_card($id_boss)
		{
			$sql = 'select id_card from _advent_enemies where id = ? limit 1';
			$sth = db::$pdo->prepare($sql);
			$sth->execute([$id_boss]);
			return $sth->fetch(PDO::FETCH_NUM)[0];
		}
		
		// узнать реальный уникальный идентификатор босса по id типа и типу
		public static function get_id_boss_by_type_a_id_type($id_boss_type, $type)	
		{
			$sql = 'select id from _advent_enemies where id_type = ? and type = ? limit 1';
			$sth = db::$pdo->prepare($sql);
			$sth->execute([$id_boss_type, $type]);
			return $sth->fetch(PDO::FETCH_NUM)[0];
		}
		
		// узнать реальный уникальный идентификатор босса по ключу (health, power...)
		public static function get_id_boss_by_key($id_user, $type)
		{
			$id_type = self::get_user_etap_boss_type($id_user, $type);
			return self::get_id_boss_by_type_a_id_type($id_type, $type);
		}
		
		// узнать начиная с каких статов возможен удачный исход сражения по id босса
		public static function get_need_param($id_boss)
		{
			$sql = 'select need_param from _advent_enemies where id = ? limit 1';
			$sth = db::$pdo->prepare($sql);
			$sth->execute([$id_boss]);
			return $sth->fetch(PDO::FETCH_NUM)[0];
		}
		
		// узнать сложность босса исходя из статов (основного)
		public static function boss_complexity_by_stat($user_stat, $need_stat)
		{
			if ($user_stat < $need_stat) {
				return 'unreal'; // нереально
			}
			$bcp = self::boss_complexity_mm_param();
			$p = (int) round(percentage($need_stat, $bcp['easy']));
			if ($user_stat >= ($need_stat + $p)) {
				return 'easy'; // легко
			}
			$p = (int) round(percentage($need_stat, $bcp['middle']));
			if ($user_stat >= ($need_stat + $p)) {
				return 'middle'; // средне
			}
			$p = (int) round(percentage($need_stat, $bcp['difficult']));
			if ($user_stat >= ($need_stat + $p)) {
				return 'difficult'; // сложно
			}
		}
		
		public static function boss_complexity_mm_param()
		{
			return [ // проценты (такие или больше статы нужны (основной)(стат + процент от этого стата) чтобы была определонная сложность)
				'easy' => 75,
				'middle' => 30,
				'difficult' => 0
			];
		}
		
		public static function boss_complexity_trans()
		{
			return [
				'unreal' => 'Нереально',
				'difficult' => 'Сложно',
				'middle' => 'Средне',
				'easy' => 'Легко'
			];
		}
		
		// узнать доступное число боссов с которыми можно сразиться
		public static function boss_avail_num($id_user)
		{
			$count = 0;
			$etaps = self::get_user_etap_boss_type_all($id_user);
			$time_rec_left = [
				'health' => self::time_recovery_left($id_user, 'health'),
				'power' => self::time_recovery_left($id_user, 'power'),
				'agil' => self::time_recovery_left($id_user, 'agil'),
				'intell' => self::time_recovery_left($id_user, 'intell')
			];
			if ( ! is_null($etaps['health']) && $time_rec_left['health'] < 1) { // если данный тип боссов еще не пройден, время восстановления прошло и боёв на зачисле проведено достаточно
				$count++;
			}
			if ( ! is_null($etaps['power']) && $time_rec_left['power'] < 1) { // если данный тип боссов еще не пройден, время восстановления прошло и боёв на зачисле проведено достаточно
				$count++;
			}
			if ( ! is_null($etaps['agil']) && $time_rec_left['agil'] < 1) { // если данный тип боссов еще не пройден, время восстановления прошло и боёв на зачисле проведено достаточно
				$count++;
			}
			if ( ! is_null($etaps['intell']) && $time_rec_left['intell'] < 1) { // если данный тип боссов еще не пройден, время восстановления прошло и боёв на зачисле проведено достаточно
				$count++;
			}
			return $count;
		}
		
		// все приключения пройдены
		public static function advent_end($id_user)
		{
			$etaps = self::get_user_etap_boss_type_all($id_user);
			if (in_array_type($etaps, 'string')) {
				return false;
			}
			return true;
		}
		
		// сколько времени осталось до отката
		public static function time_recovery_left($id_user, $type)
		{
			$sql = "select {$type}_time from users_advent where id_user = ? limit 1";
			$sth = db::$pdo->prepare($sql);
			$sth->execute([$id_user]);
			$t = $sth->fetch(PDO::FETCH_NUM)[0];
			return ($t + self::TIME_RECOVERY) - time();
		}
		
		// получить id самого последнего босса определенной категории
		public static function get_last_id_boss_by_type($type)
		{
			$put = '(select max(id_type) from _advent_enemies where type = ?)';
			$sql = 'select id from _advent_enemies where id_type = %s and type = ? limit 1';
			$sql = sprintf($sql, $put);
			$sth = db::$pdo->prepare($sql);
			$sth->execute([$type, $type]);
			return $sth->fetch(PDO::FETCH_NUM)[0];
		}
		
		// получает сложность босса (англ.) и в зависимости от этого делает расчет: победа или поражение, возвращает true или false соответственно
		public static function victory_or_lost($id_user, $complex)
		{
			if ($complex == 'unreal') {
				return false;
			}
			$chance = mt_rand(1, 100);
			if ($complex == 'easy') {
				if ($chance <= 79) {
					return true;
				}
				else return false;
			}
			if ($complex == 'middle') {
				if ($chance <= 30) return true;
				else return false;
			}
			if ($complex == 'difficult') {
				if ($chance <= 4) return true;
				else return false;
			}
		}
		
		// изменить значение этапа определенного типа боссов
		// параметр set - число на которое нужно увелечить этап, если задать null, то поле обnullится
		public static function etap_update($id_user, $type, $set = 1)
		{
			$sql = 'update users_advent set %s where id_user = ? limit 1';
			if (is_null($set)) {
				$sql = sprintf($sql, "$type = null");
				$sth = db::$pdo->prepare($sql);
				$sth->execute([$id_user]);
			}
			else {
				$sql = sprintf($sql, "$type = $type + ?");
				$sth = db::$pdo->prepare($sql);
				$sth->execute([$set, $id_user]);
			}
			/* v($id_user);
			v($type);
			v($set);
			die; */
		}
		
		// записать новое время определенному типу босса
		public static function boss_type_time_update($id_user, $type, $time = false)
		{
			if ($time === false) $time = time();
			$sql = 'update users_advent set %s = ? where id_user = ? limit 1';
			$sql = sprintf($sql, "{$type}_time");
			$sth = db::$pdo->prepare($sql);
			$sth->execute([$time, $id_user]);
		}
		
		// установить результат боя
		public static function set_result($id_user, $result)
		{
			$sql = 'update users_dyn set result_advent = ? where id_user = ? limit 1';
			$sth = db::$pdo->prepare($sql);
			$sth->execute([$result, $id_user]);
		}
		
		// установить результат боя
		public static function get_result($id_user)
		{
			$sql = 'select result_advent from users_dyn where id_user = ? limit 1';
			$sth = db::$pdo->prepare($sql);
			$sth->execute([$id_user]);
			return $sth->fetch(PDO::FETCH_NUM)[0];
		}
		
		// узнать награду опыта по id босса
		public static function get_exp_reward_by_id_boss($id_boss)
		{
			$sql = 'select exp_reward from _advent_enemies where id = ? limit 1';
			$sth = db::$pdo->prepare($sql);
			$sth->execute([$id_boss]);
			return $sth->fetch(PDO::FETCH_NUM)[0];
		}
		
		// узнать награду  в баксах по id босса
		public static function get_bucks_reward_by_id_boss($id_boss)
		{
			$sql = 'select bucks_reward from _advent_enemies where id = ? limit 1';
			$sth = db::$pdo->prepare($sql);
			$sth->execute([$id_boss]);
			return $sth->fetch(PDO::FETCH_NUM)[0];
		}
		
		// узнать награду  в мутагене по id босса
		public static function get_mutagen_reward_by_id_boss($id_boss)
		{
			$sql = 'select mutagen_reward from _advent_enemies where id = ? limit 1';
			$sth = db::$pdo->prepare($sql);
			$sth->execute([$id_boss]);
			return $sth->fetch(PDO::FETCH_NUM)[0];
		}
		
		// цвета сложностей (классы css)
		public static function get_complexity_colors()
		{
			return [
				'easy' => '#5BA100',
				'middle' => '#CA9100',
				'difficult' => '#E33011',
				'unreal' => '#8D1717'
			];
		}
	}




	class model_advent
	{
		public function get($type)
		{
			$user_id = ID;
			
			$is_completed = \db\users_advent::is_completed($user_id, $type);
			$etap = advent::get_user_etap_boss_type($user_id, $type);
			if ($is_completed) {
				$boss_real_id = $etap;
			}
			else {
				$boss_real_id = advent::get_id_boss_by_type_a_id_type($etap, $type);
			}
			$boss_card_id = advent::get_id_card($boss_real_id);
			$boss_name = advent::get_boss_name_by_id($boss_real_id);
			
			
			$user_param = uc::get_param_all($user_id);
			$param_name = translate::param_name($type);
			$bcmmp = advent::boss_complexity_mm_param();
			$param_need = advent::get_need_param($boss_real_id); // нужные статы
			$param_recom = (int) $param_need + round(percentage($param_need, $bcmmp['middle']));
			$boss_complexity = advent::boss_complexity_by_stat($user_param[$type], $param_need); // сложность
			
			$colors = advent::get_complexity_colors();
			$complex_color = $colors[$boss_complexity];
			
			// транслейт сложностей
			$bct = advent::boss_complexity_trans();
			$complex = $bct[$boss_complexity];
			$recoil = advent::time_recovery_left($user_id, $type); // остаток секунд до отката
			$recoil_all = [
				'health' =>advent::time_recovery_left($user_id, 'health'),
				'power' =>advent::time_recovery_left($user_id, 'power'),
				'agil' =>advent::time_recovery_left($user_id, 'agil'),
				'intell' =>advent::time_recovery_left($user_id, 'intell')
			];
			
			$exodus = advent::get_result($user_id); // исход битвы
			
			$mutagen_gather_max = advent_mutagen::user_mutagen_max($user_id);
			$mutagen_collected = db\users_advent::mutagen_collected_get($user_id);
			$mutagen_param_sum = advent_mutagen::MUTAGEN_PARAM_SUM;
			$mutagen_num = advent_mutagen::MUTAGEN_NUM;
			
			$param_rus_what = [
				'health' => 'выносливости',
				'power' => 'силы',
				'agil' => 'ловкости',
				'intell' => 'интеллекта'
			];
			
			return [
				'type' => $type,
				'is_completed' => $is_completed,
				'exodus' => $exodus,
				'etap' => $etap,
				'boss_name' => $boss_name,
				'boss_card_id' => $boss_card_id,
				'complex_color' => $complex_color,
				'param_recom' => $param_recom,
				'recoil' => $recoil,
				'recoil_all' => $recoil_all,
				'param_name' => $param_name,
				'mutagen_gather_max' => $mutagen_gather_max,
				'mutagen_collected' => $mutagen_collected,
				'mutagen_param_sum' => $mutagen_param_sum,
				'mutagen_num' => $mutagen_num,
				'param_rus_what' => $param_rus_what
			];
		}
	}
}