<?php

namespace indrat {
	class cards extends \_tbl_line
	{
		protected static $table = '_cards';
		
		// получить имя карточки
		public static function card_name_get($card_id)
		{
			return static::get('name', 'where id = ?', [$card_id]);
		}
		
		// получить качество карточки
		public static function card_quality_get($card_id)
		{
			return static::get('quality', 'where id = ?', [$card_id]);
		}
		
		// получить параметры карточки
		public static function card_param_get($card_id)
		{
			return static::get('health, power, agil, intell : assoc', 'where id = ?', [$card_id]);
		}
		
		// получить основной параметр карточки (тот который самый большой)
		public static function card_param_main_get($card_id)
		{
			$card_param = self::card_param_get($card_id);
			$card_type = 'health';
			if ($card_param['health'] > 0) $card_type = 'health';
			if ($card_param['power'] > 0) $card_type = 'power';
			if ($card_param['agil'] > 0) $card_type = 'agil';
			if ($card_param['intell'] > 0) $card_type = 'intell';
			return [$card_type => $card_param[$card_type]];
		}
		
		// получить максимально возможное число таких карточек в коллекции у игроков
		public static function card_max_get($card_id)
		{
			return static::get('max', 'where id = ?', [$card_id]);
		}
		
		// цена карточки в магазине (в мутагене)
		public static function card_price_get($card_id)
		{
			return static::get('price_mutagen', 'where id = ?', [$card_id]);
		}
		
		// посчитать карточки
		public static function cards_count()
		{
			return static::get('count(*)');
		}
		
		// генерировать случайный id-карточки
		public static function card_id_random($num = 1)
		{
			$cards_num = self::cards_count() - 1;
			for ($i = 0; $i < $num; $i++) {
				$rand = mt_rand(0, $cards_num);
				$ids[] = static::get('id', ['', "limit $rand, 1"]);
			}
			if (count($ids) == 1) {
				return $ids[0];
			}
			return $ids;
		}
		
		// генерировать массив со случайными id карточек
		public static function card_id_norepeat_random($num)
		{
			$cards = self::cards_id_all_get();
			$key_rand = array_rand($cards, $num);
			foreach ($key_rand as $key) {
				$card_keys[] = $cards[$key][0];
			}
			return $card_keys;
		}
		
		// генерировать id-карточки по качеству
		public static function card_id_quality_random($quality)
		{
			$cards = self::cards_id_quality_get($quality);
			return $cards[array_rand($cards)][0];
		}
		
		// получить id всех карточек
		public static function cards_id_all_get()
		{
			$sql = 'select id from _cards';
			$sth = \db::$pdo->query($sql);
			return $sth->fetchAll(\PDO::FETCH_NUM);
		}
		
		// получить id всех карточек определенного качества
		public static function cards_id_quality_get($quality)
		{
			$sql = "select id from _cards where quality = $quality";
			$sth = \db::$pdo->query($sql);
			return $sth->fetchAll(\PDO::FETCH_NUM);
		}
		
		// получить тип бонуса коллекции
		public static function collect_bonus_type_get($card_id)
		{
			return self::get('collect_bonus_type', 'where id = ?', [$card_id]);
		}
		
		// получить бонус коллекции
		public static function collect_bonus_get($card_id)
		{
			return self::get('collect_bonus', 'where id = ?', [$card_id]);
		}
	}

	class user_cards extends \_tbl_line
	{
		protected static $table = 'users_cards';
		
		// получить запись с карточками (сериализованный массив)
		public static function cards_get($user_id = false)
		{
			user_id($user_id);
			return static::get('cards', 'where id_user = ?', [$user_id]);
		}
		
		// получить массив с карточками (десериализованый),
		// где ключ, это id карточки в _cards, а значение количество таких карточек у игрока, если карточек нет, возвратит false
		public static function cards_conv_get($user_id = false)
		{
			user_id($user_id);
			$cards = unserialize(self::cards_get($user_id));
			if ( ! $cards) {
				return [];
			} return $cards;
		}
		
		// получить количество карточек с конкретным id
		public static function card_num_get($card_id, $user_id = false)
		{
			user_id($user_id);
			$cards = self::cards_conv_get($user_id);
			return (array_key_exists($card_id, $cards)) ? $cards[$card_id] : 0;
		}
		
		// проверить существует ли карточки с таким id у игрока
		public static function card_is($card_id, $user_id = false)
		{
			user_id($user_id);
			$cards = self::cards_conv_get($user_id);
			if ( ! is_array($cards)) {
				return false;
			}
			if (array_key_exists($card_id, $cards)) {
				return true;
			} return false;
		}
		
		// записать десиарелизованый массив карточек
		public static function cards_conv_set($cards_array, $user_id = false)
		{
			user_id($user_id);
			$cards = serialize($cards_array);
			static::set('cards = ?', 'where id_user = ?',[$cards, $user_id]);
		}
		
		// добавить новую карточку (создать)
		public static function card_new($card_id, $user_id = false)
		{
			user_id($user_id);
			$cards = self::cards_conv_get($user_id);
			$cards[$card_id] = 1;
			self::cards_conv_set($cards, $user_id);
		}
		
		// изменить количество карточек
		public static function card_num_update($card_id, $num, $user_id = false)
		{
			user_id($user_id);
			$cards = self::cards_conv_get($user_id);
			$cards[$card_id] += $num;
			self::cards_conv_set($cards, $user_id);
		}
		
		// вернет true, если коллекция таких карточек уже собрана (т.е. нельзя получить больше таких карточек)
		// если указан параметр $num, то эта функция работает как "перебор", т.е. вернет true если при добавлении игроку такого кол-ва карточек будет перебор
		public static function cards_collected($card_id, $user_id = false, $num = 0)
		{
			user_id($user_id);
			$cards_num = self::card_num_get($card_id, $user_id);
			$cards_max = \cards::card_max_get($card_id);
			if ($num > 0) {
				if (($cards_num + $num) > $cards_max) {
					return true;
				} return false;
			}
			if ($cards_num >= $cards_max) {
				return true;
			} return false;
		}
		
		// получить идентификаторы карточек, которых собрана коллекция (макс.)
		public static function cards_collected_get($user_id)
		{
			$cards = self::cards_conv_get($user_id);
			$cards_coll = [];
			foreach ($cards as $card_id => $card_num) {
				$card_max = cards::card_max_get($card_id);
				if ($card_num >= $card_max) {
					$cards_coll[] = $card_id;
				}
			}
			return $cards_coll;
		}
		
		// получить общий бонус к $bonus_type от всех собранных коллекций
		public static function collected_bonus_get($user_id, $bonus_type)
		{
			$cards_coll = self::cards_collected_get($user_id);
			if ( ! $cards_coll) {
				return 0;
			}
			$cards_coll_str = implode(',', $cards_coll);
			$sql = "select sum(collect_bonus) from _cards where id in ($cards_coll_str) and collect_bonus_type = ? limit 1";
			$sth = \db::$pdo->prepare($sql);
			$sth->execute([$bonus_type]);
			$bonus = $sth->fetchColumn();
			return is_null($bonus) ? 0 : $bonus;
		}
	}
}

namespace { // глобально
	class cards extends \indrat\cards
	{
		
	}
	
	class user_cards extends \indrat\user_cards
	{
		public static function card_new($card_id, $user_id = false)
		{
			user_id($user_id);
			parent::card_new($card_id, $user_id);
			$param = cards::card_param_get($card_id);
			uc::update_param_all($user_id, $param);
		}
		
		public static function card_num_update($card_id, $num, $user_id = false)
		{
			user_id($user_id);
			$param = cards::card_param_get($card_id);
			foreach ($param as $key => $val) {
				$param[$key] = $val * $num;
			}
			$minus = ($num < 0) ? true : false;
			uc::update_param_all($user_id, $param, $minus); // изменить параметры игрока
			parent::card_num_update($card_id, $num, $user_id); // увелить количество карточек на $num
		}
		
		// контроллер получения карточки
		public static function card_up_ctrl($card_id, $num, $user_id = false)
		{
			user_id($user_id);
			if (self::cards_collected($card_id, $user_id, $num)) { // при получении этой карточки будет перебор в коллекции
				return;
			}
			if (static::card_is($card_id, $user_id)) {
				static::card_num_update($card_id, $num, $user_id);
			}
			else {
				static::card_new($card_id, $user_id);
				if ($num > 1) { // Z:тут всё хорошо !
					static::card_num_update($card_id, ($num - 1), $user_id);
				}
			}
			dyn_card_received_set($card_id, $user_id);
			
			$card_num = self::card_num_get($card_id, $user_id);
			$card_max = cards::card_max_get($card_id);
			if ($card_num >= $card_max) { // если будет собрана коллекция
				$coll_bonus_type = cards::collect_bonus_type_get($card_id);
				$coll_bonus = cards::collect_bonus_get($card_id);
				$perhaps = ['health', 'power', 'agil', 'intell'];
				if (in_array($coll_bonus_type, $perhaps)) {
					uc::update_param($user_id, $coll_bonus_type, $coll_bonus);
				}
				$card_name = cards::card_name_get($card_id);
				$bonuses_rus = [
					'expr' => 'Опыт',
					'bucks' => 'Баксы',
					'metal' => 'Металлолом',
					'mutagen_advent' => 'Мутаген в Приключениях'
				];
				$bonuses_rus += \translate::param_name();
				$percent = '';
				if ($coll_bonus_type == 'expr' || $coll_bonus_type == 'bucks' || $coll_bonus_type == 'metal') {
					$percent = '%';
				}
				$jdata = [
					'title' => 'Собрана коллекция!',
					'text' => "Вы собрали коллекцию карточек <b>$card_name</b>. $bonuses_rus[$coll_bonus_type] <span class='limeGreen'>+$coll_bonus$percent</span>"
				];
				journal::record_add($jdata, $user_id);
			}
		}
	}
	
	class cards_model
	{
		public function collect($user_id)
		{
			$cards = user_cards::cards_conv_get($user_id);
			$temp = [];
			foreach ($cards as $card_id => $card_num) {
				$card_param = cards::card_param_get($card_id);
				$card_type = 'power';
				if ($card_param['health'] > 0) $card_type = 'health';
				if ($card_param['power'] > 0) $card_type = 'power';
				if ($card_param['agil'] > 0) $card_type = 'agil';
				if ($card_param['intell'] > 0) $card_type = 'intell';
				$temp[] = [
					'id' => $card_id,
					'num' => $card_num,
					'max' => cards::card_max_get($card_id),
					'name' => cards::card_name_get($card_id),
					'param' => $card_param,
					'param_sum' => $card_param[$card_type] * $card_num,
					'type' => $card_type,
					'type_rus' => \translate::param_name($card_type),
					'quality' => cards::card_quality_get($card_id),
					'collect_bonus_type' => cards::collect_bonus_type_get($card_id),
					'collect_bonus' => cards::collect_bonus_get($card_id)
				];
			}
			$af = function ($a, $b) {
				if ($a['quality'] == $b['quality']) {
					return 0;
				}
				return ($a['quality'] < $b['quality']) ? 1 : -1;
			};
			usort($temp, $af);
			$num = count($temp);
			
			$pag = new pagination;
			$limit = $pag->start($num, 10);
			$cards = array_slice($temp, $limit['start'], $limit['limit']);
			$collect_bonus_type_rus = \translate::param_name();
			$collect_bonus_type_rus = $collect_bonus_type_rus + [
				'expr' => 'Опыт',
				'bucks' => 'Баксы',
				'metal' => 'Металлолом',
				'mutagen_advent' => 'Мутаген в Приключениях'
			];
			return [
				'user_id' => $user_id,
				'pag' => $pag,
				'cards' => $cards,
				'collect_bonus_type_rus' => $collect_bonus_type_rus
			];
		}
	}
}