<?php

/**
 * клановая локация 1 (битва шредера)
 */

class cl1
{
	const CL1_RECOVERY_DAYS = 1; // время восстанавления локации (в сутках)
	const CL1_PASSING_DAYS = 1; // время прохождения локации (в сутках)
	
	const USER_BATTLES_DEFAULT = 20; // максимальное количество боёв
	const USER_BATTLES_RECOVERY_SECONDS_INTERVAL = 900; // с каким интервалом восстанавливаются бои у игрока
	const USER_BATTLES_RECOVERY_NUM = 5;
	const USER_REWARD_EXP_BASE = 80; // базовое количество опыта за один бой (удар)
	
	
	// стадия
	public static function loc_get_stage()
	{
		$sql = 'select stage from cl1 limit 1';
		$sth = db::$pdo->query($sql);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// достать время локации
	public static function loc_get_stage_timestamp()
	{
		$sql = 'select stage_timestamp from cl1 limit 1';
		$sth = db::$pdo->query($sql);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// узнать остаток времени до восстанавления локации
	public static function loc_time_recovery_left()
	{
		return left_seconds_midnight(self::CL1_RECOVERY_DAYS);
	}
	
	// узнать остаток времени до завершения прохождения локации
	public static function loc_time_passing_left()
	{
		return left_seconds_midnight(self::CL1_PASSING_DAYS);
	}
	
	// прошло ли время восстанавления локации
	public static function loc_time_recovery_gone()
	{
		if (day_new(self::loc_get_stage_timestamp(), self::CL1_RECOVERY_DAYS)) { // если количество установленных дней для восстанавления локации прошло
			return true;
		}
		return false;
	}
	
	// прошло ли время прохождения локации
	public static function loc_time_passing_gone()
	{
		if (day_new(self::loc_get_stage_timestamp(), self::CL1_RECOVERY_DAYS)) { // если количество установленных дней для прохождения локации прошло
			return true;
		}
		return false;
	}
	
	
	
	// изменить стадию локации
	public static function loc_update_stage($stage)
	{
		$sql = "update cl1 set stage = ? limit 1";
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$stage]);
	}
	
	// установить новое время стадии
	public static function loc_set_stage_timestamp($time = false)
	{
		if ($time === false) {
			$time = time();
		}
		$sql = "update cl1 set stage_timestamp = ? limit 1";
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$time]);
	}
	
	
	
	
	
	// существует ли запись с игроком у конкретного клана
	public static function is_user($user_id, $clan_id)
	{
		$sql = 'select count(*) from cl1_part where id_clan = ? and id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$clan_id, $user_id]);
		if ($sth->fetch(PDO::FETCH_NUM)[0] > 0) {
			return true;
		} return false;
	}
	
	// получить счёт игрока
	public static function get_user_score($user_id)
	{
		$sql = 'select score from cl1_users where id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$user_id]);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// получить последний счёт игрока
	public static function get_user_score_last($user_id)
	{
		$sql = 'select score_last from cl1_users where id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$user_id]);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// записать последний счёт игрока
	public static function set_user_score_last($user_id, $score)
	{
		$sql = 'update cl1_users set score_last = ? where id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$score, $user_id]);
	}
	
	// получить счёт игрока у конкретного клана
	public static function get_user_score_in_clan($user_id, $clan_id)
	{
		$sql = 'select score from cl1_part where id_clan = ? and id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$clan_id, $user_id]);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// получить общий счёт (всех кланов вместе взятых)
	public static function get_score_overall()
	{
		// $sql = 'select sum(cl1p.score) from cl1_part cl1p where cl1p.id_clan = (select u.id_clan_last from users u where u.id = cl1p.id_user limit 1)  limit 1';
		$sql = 'select sum(score) from cl1_clan limit 1';
		$sth = db::$pdo->query($sql);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// получить общий счёт одного из кланов
	public static function get_score_clan($clan_id)
	{
		$sql = 'select score from cl1_clan where id_clan = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$clan_id]);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// получить процент счета клана относительно общего счёта
	public static function get_score_clan_percent($clan_id)
	{
		$overall = self::get_score_overall();
		$clan = self::get_score_clan($clan_id);
		return compercent($clan, $overall);
	}
	
	// получить процент счёта игрока относительно клана
	// если clan_id не задан, то относительно того клана в котором нахаодится
	public static function get_user_score_percent_in_clan($user_id, $clan_id = false)
	{
		if ($clan_id === false) {
			$clan_id = clan::get_id_clan_by_id_user($user_id);
		}
		$clan_score = self::get_score_clan($clan_id);
		$user_score = self::get_user_score_in_clan($user_id, $clan_id);
		return compercent($user_score, $clan_score);
	}
	
	// получить процент счёта игрока относительно всех игроков
	public static function get_user_score_percent_in_overall($user_id)
	{
		$clan_id = clan::get_id_clan_by_id_user($user_id);
		$score_overall = self::get_score_overall();
		$user_score = self::get_user_score($user_id, $clan_id);
		return compercent($user_score, $score_overall);
	}
	
	// получить количество боёв у игрока
	public static function get_user_battles($user_id)
	{
		$sql = 'select battles from cl1_users where id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$user_id]);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// достать время восстанавления боёв игрока
	public static function get_user_battles_recovery_timestamp($user_id)
	{
		$sql = 'select battles_recovery_timestamp from cl1_users where id_user = ?';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$user_id]);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// узнать остаток времени до восстанавления боёв игрока
	public static function get_user_battles_recovery_seconds_left($user_id)
	{
		return (self::get_user_battles_recovery_timestamp($user_id) + self::USER_BATTLES_RECOVERY_SECONDS_INTERVAL) - time();
	}
	
	// узнать прошло ли время восстанавления боёв игрока
	public static function user_battles_recovery_time_gone($user_id)
	{
		$battles_rec_time = self::get_user_battles_recovery_timestamp($user_id);
		if (utime_new($battles_rec_time, self::USER_BATTLES_RECOVERY_SECONDS_INTERVAL)) {
			return true;
		}
		return false;
	}
	
	// достать id участников локации в общем списке
	// zerofilter = true пропускает нулевые значения
	public static function get_part_overall($limit = false, $start = false, $zerofilter = false, $ordby = false)
	{
		$zf = ''; $l = ''; $ob = 'order by score desc';
		if ($zerofilter === true) {
			$zf = 'where score > 0';
		}
		if ($limit !== false) {
			$l = "limit $limit";
		}
		if ($start !== false) {
			$l = "limit $start, $limit";
		}
		if ($ordby !== false) {
			$ob = $ordby;
		}
		// $ins = 'select u.id_clan_last from users u where u.id = cl1p.id_user limit 1';
		// $sql = 'select cl1p.id_user from cl1_part cl1p where cl1p.id_clan = (%s) %s %s %s';
		$sql = 'select id_user from cl1_users %s %s %s';
		$sql = sprintf($sql, $zf, $ob, $l);
		$sth = db::$pdo->prepare($sql);
		$sth->execute();
		return $sth->fetchAll(PDO::FETCH_NUM);
	}
	
	
	// достать id участников локации в списке клана
	// zerofilter = true пропускает нулевые значения
	public static function get_part_clan($id_clan, $limit = false, $start = false, $zerofilter = false, $ordby = false)
	{
		$zf = ''; $l = ''; $ob = 'order by score desc';
		if ($zerofilter === true) {
			$zf = 'and score > 0';
		}
		if ($limit !== false) {
			$l = "limit $limit";
		}
		if ($start !== false) {
			$l = "limit $start, $limit";
		}
		if ($ordby !== false) {
			$ob = $ordby;
		}
		$sql = 'select id_user from cl1_part where id_clan = ?  %s %s %s';
		$sql = sprintf($sql, $zf, $ob, $l);
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_clan]);
		return $sth->fetchAll(PDO::FETCH_NUM);
	}
	
	// очки которые начисляются за один удар
	public static function score_one($id_user)
	{
		return fighting_score::generate($id_user);
	}
	
	// генерировать награду в зависимости от общего счета клана
	public static function reward_generate($score)
	{
		return [
			'bucks' => (int) round($score / 6),
			'mutagen' => (int) ceil($score / 5000),
			'metal' => (int) round($score / 9)
		];
	}
	
	// генерировать опыт за удар
	public static function exp_generate()
	{
		$min = floor(self::USER_REWARD_EXP_BASE / 1.15);
		$max = ceil(self::USER_REWARD_EXP_BASE * 1.15);
		return mt_rand($min, $max);
	}
	
	// узнать сумму очков всего клана
	// псевдоним
	public static function get_clan_score_sum($id_clan)
	{
		return self::get_score_clan($id_clan);
	}
	
	// достать id кланов локации
	public static function get_clans($limit = false, $start = false, $zerofilter = false, $ordby = false)
	{
		$zf = ''; $l = ''; $ob = 'order by score desc';
		if ($zerofilter === true) {
			$zf = 'where score > 0';
		}
		if ($limit !== false) {
			$l = "limit $limit";
		}
		if ($start !== false) {
			$l = "limit $start, $limit";
		}
		if ($ordby !== false) {
			$ob = $ordby;
		}
		$sql = 'select id_clan from cl1_clan %s %s %s';
		$sql = sprintf($sql, $zf, $ob, $l);
		$sth = db::$pdo->query($sql);
		return $sth->fetchAll(PDO::FETCH_ASSOC);
	}
	
	// узнать статус награды клана
	public static function get_clan_reward_status($id_clan)
	{
		$sql = 'select reward_status from cl1_clan where id_clan = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_clan]);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// получить значение награды клана
	public static function get_clan_reward($id_clan)
	{
		$sql = 'select reward_bucks, reward_mutagen, reward_metal from cl1_clan where id_clan = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_clan]);
		$reward = $sth->fetch(PDO::FETCH_ASSOC);
		return [
			'bucks' => $reward['reward_bucks'],
			'mutagen' => $reward['reward_mutagen'],
			'metal' => $reward['reward_metal']
		];
	}
	
	// достаточно ли у игрока привилегий чтобы забирать награду клана
	public static function is_user_get_reward_privilege($id_user)
	{
		$user_rank = clan::get_user_clan_rank($id_user);
		$privilege = package('privilege-cl1-get-reward');
		return $privilege[$user_rank];
	}
	
	// посчитать все кланы участники
	public static function clans_count()
	{
		$sql = 'select count(distinct id_clan) from cl1_part where score > 0  limit 1';
		$sth = db::$pdo->query($sql);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// посчитать всех участников
	public static function part_count()
	{
		$sql = 'select count(distinct id_user) from cl1_part where score > 0 limit 1';
		$sth = db::$pdo->query($sql);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// посчитать всех участников какого-то клана
	public static function clan_part_count($clan_id)
	{
		$sql = 'select count(*) from cl1_part where id_clan = ? and score > 0 limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$clan_id]);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	
	
	
	// создать запись с игроком
	public static function user_create($user_id, $clan_id)
	{
		$sql = 'insert into cl1_part set id_clan = ?, id_user = ?';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$clan_id, $user_id]);
	}
	
	// изменить количество очков игрока
	public static function user_score_update($user_id, $score)
	{
		$sql = 'update cl1_users set score = score + ? where id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$score, $user_id]);
	}
	
	// изменить количество очков игрока
	public static function user_score_in_clan_update($user_id, $clan_id, $score)
	{
		$sql = 'update cl1_part set score = score + ? where id_clan = ? and id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$score, $clan_id, $user_id]);
	}
	
	public static function clan_score_update($clan_id, $score)
	{
		$sql = 'update cl1_clan set score = score + ? where id_clan = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$score, $clan_id]);
	}
	
	// удалить всех участников (почистить таблицу cl1_part (truncate))
	public static function part_truncate()
	{
		$sql = 'truncate table cl1_part';
		db::$pdo->exec($sql);
	}
	
	// обнулить счёты кланов
	public static function clans_back()
	{
		$sql = 'update cl1_clan set score = 0 where score > 0';
		db::$pdo->exec($sql);
	}
	
	// обнулить счёты игроков
	public static function users_back()
	{
		$sql = 'update cl1_users set score = 0, battles_count = 0 where score > 0 or battles_count > 0';
		db::$pdo->exec($sql);
	}
	
	// изменить количество боёв игрока
	public static function battles_update($user_id, $num, $rewrite = false)
	{
		$o = 'battles = battles + ?';
		if ($rewrite) {
			$o  = 'battles = ?';
		}
		$sql = "update cl1_users set $o where id_user = ? limit 1";
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$num, $user_id]);
	}
	
	// увеличить значение счётчика игрока (участие)
	public static function battles_count_update($user_id, $num)
	{
		$sql = 'update cl1_users set battles_count = battles_count + ? where id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$num, $user_id]);
	}
	
	// получить значение счётчика игрока (участие)
	public static function battles_count_get($user_id)
	{
		$sql = 'select battles_count from cl1_users where id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$user_id]);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// изменить метку времени восстанавления боёв игрока
	public static function battles_recovery_timestamp_update($user_id, $time = false)
	{
		if ($time === false) {
			$time = time();
		}
		$sql = "update cl1_users set battles_recovery_timestamp = ? where id_user = ? limit 1";
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$time, $user_id]);
	}
	
	// начислить награду клану (во временный столбец)
	// $reward - массив ['bukcs' => ?, 'mutagen' => ?]
	// $rewrite = true - перезаписать награду, а не увеличить
	public static function clan_reward_update($id_clan, $reward, $rewrite = false)
	{
		$f = 'reward_bucks = reward_bucks + ?, reward_mutagen = reward_mutagen + ?, reward_metal = reward_metal + ?';
		if ($rewrite) {
			$f = 'reward_bucks = ?, reward_mutagen = ?, reward_metal = ?';
		}
		$sql = 'update cl1_clan set %s where id_clan = ? limit 1';
		$sql = sprintf($sql, $f);
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$reward['bucks'], $reward['mutagen'], $reward['metal'], $id_clan]);
	}
	
	// изменить статус награды
	public static function clan_reward_status_update($id_clan, $status)
	{
		$sql = 'update cl1_clan set reward_status = ? where id_clan = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$status, $id_clan]);
	}
	
	// контроллер создания записи с игроком
	public static function user_ctrl()
	{
		$user_id = ID;
		if (self::loc_get_stage() != 2) { // если время восстановления локации еще не прошло
			return false;
		}
		if ( ! clan::is_user_clan($user_id)) { // если игрок не в клане
			return false;
		}
		$clan_id = clan::get_id_clan_by_id_user($user_id);
		if (self::is_user($user_id, $clan_id)) { // если запись с игроком уже есть у этого клана
			return false;
		}
		self::user_create($user_id, $clan_id);
	}
	
	// контроллер сражения
	public static function battle_ctrl()
	{
		$user_id = ID;
		if (self::loc_get_stage() != 2) { // если не сражение
			return false;
		}
		if ( ! clan::is_user_clan($user_id)) { // если игрок не в клане
			return false;
		}
		if (cl1::get_user_battles($user_id) < 1) { // если боёв нет
			return false;
		}
		$clan_id = clan::get_id_clan_by_id_user($user_id);
		$score_one = self::score_one($user_id);
		self::user_score_update($user_id, $score_one);
		self::user_score_in_clan_update($user_id, $clan_id, $score_one);
		self::clan_score_update($clan_id, $score_one);
		self::battles_update($user_id, -1);
		self::battles_count_update($user_id, 1);
		self::set_user_score_last($user_id, $score_one);
		$bcount = self::battles_count_get($user_id);
		
		$user_level = \user::level_get($user_id);
		$expr = self::exp_generate();
		$expr += $user_level * 13;
		$obj_expr = new \expr($user_id);
		$obj_expr->base = $expr;
		$obj_expr->add();
		
		// шанс найти мутаген и метал зависит от уровня игрока
		/* $mutagen_chance_percent = ceil($user_level / 2);
		$mutagen_chance_percent = ($mutagen_chance_percent <= 20) ? $mutagen_chance_percent : 20; */
		$metal_chance_percent = ceil($user_level / 1.5);
		$metal_chance_percent = ($metal_chance_percent <= 30) ? $metal_chance_percent : 30;
		
		$currency = new \currency;
		$currency->user_id = $user_id;
		/* if (mt_rand(1, 100) <= $mutagen_chance_percent) {
			$currency->base = 1;
			$currency->mutagen_add();
		} */
		if (mt_rand(1, 100) <= $metal_chance_percent) {
			$metal = mt_rand(30, 50);
			$metal += $user_level * 5;
			$currency->base = $metal;
			$currency->metal_add();
		}
		
		$user_statistic = new \user_statistic($user_id);
		$user_statistic->increase_statistic('cl1_score', $score_one);
	}
	
	// контроллер восстановления боёв у игрока
	public static function battles_recovery_ctrl()
	{
		if (self::loc_get_stage() != 2) { // если не сражение, то смысла восстанавливать нет
			return false;
		}
		$user_id = ID;
		$enemies_num_max = self::USER_BATTLES_DEFAULT;
		$enemies_recovery_num = self::USER_BATTLES_RECOVERY_NUM;
		$enemies_recovery_seconds = self::USER_BATTLES_RECOVERY_SECONDS_INTERVAL;
	
		$enemies_num_timestamp = self::get_user_battles_recovery_timestamp($user_id);
		$enemies_num = self::get_user_battles($user_id);
	
		$rec = new location_recovery;
		$rec->timestamp = $enemies_num_timestamp;
		$rec->num_current = $enemies_num;
		$rec->recovery_seconds = $enemies_recovery_seconds;
		$rec->recovery_num = $enemies_recovery_num;
		$rec->num_max = $enemies_num_max;
		$rec->tap();
		
		$timestamp = $rec->timestamp_new;
		$num = $rec->num_new;
		if ($timestamp != $enemies_num_timestamp) {
			self::battles_recovery_timestamp_update($user_id, $timestamp); // записать новое время
		}
		if ($num != $enemies_num) {
			self::battles_update($user_id, $num, true); // записать бои
		}
	}
	
	// контроллер стадии
	public static function stage_ctrl()
	{
		$stage = $stage_new = self::loc_get_stage();
		if ($stage == 1) { // стадия-1 (восстановление)
			if (self::loc_time_recovery_gone() && robbery::get_stage() == 1) { // время восстанавления прошло и ограбление закрыто
				self::loc_update_stage(2); // изменить стадию
				self::loc_set_stage_timestamp(); // установить новую ВМ
				self::part_truncate(); // очистить всю статистику/удалить всех преждних участников
				/** !!! возможно и не пригодится, self::battles_recovery_ctrl() сделает всю необходимую работу...
				null; // обновить количество боёв всех игроков и записать им всем новое время (todo:сделать другую механику данного функционала, т.к. при наличии огромного количества игроков возможно падение производительности/первый кто запустит скрипт с данным участком кода возможно будет вынужден ждать долгое время для его завершения, так же возможно осуществить данную хренотень через cron)
				*/
				self::clans_back(); // ?
				self::users_back(); // ?
				return;
			}
		}
		if ($stage == 2) { // стадия-2 (прохождение)
			if (self::loc_time_passing_gone()) { // время восстанавления прошло
				self::loc_update_stage(1); // изменить стадию
				self::loc_set_stage_timestamp(); // установить новую ВМ
				return;
			}
		}
	}
	
	// контроллер сбора награды игроком
	public static function clan_get_reward_ctrl()
	{
		$stage = self::loc_get_stage();
		if ($stage != 1) { // забирать награду можно только если стадия 1 (восстанавление)
			return false;
		}
		$id_user = ID;
		if ( ! clan::is_user_clan($id_user)) { // этот игрок без клана
			return false;
		}
		if ( ! self::is_user_get_reward_privilege($id_user)) { // у игрока недостаточно привилегий для сбора награды
			return false;
		}
		$id_clan = clan::get_id_clan_by_id_user($id_user);
		$clan_reward = self::get_clan_reward($id_clan);
		if (array_sum($clan_reward) < 1) { // награды нет
			return false;
		}
		self::clan_reward_update($id_clan, ['bucks' => -$clan_reward['bucks'], 'mutagen' => -$clan_reward['mutagen'], 'metal' => -$clan_reward['metal']]); // снять награду из временного столбца
		clan_coffers::bucks_refill($id_clan, $clan_reward['bucks']);
		clan_coffers::mutagen_refill($id_clan, $clan_reward['mutagen']);
		clan_coffers::metal_refill($id_clan, $clan_reward['metal']);
		
		notf::set_notf('Награда начислена в казну клана.', 'default');
	}
	
	public static function clans_reward()
	{
		/* 
			stage: 1 - восстанавление, stage: 2 прохождение
			reward_status: 1 - награда не начислена
			reward_status: 2 - награда начислена
		*/
		$stage = self::loc_get_stage();
		if ($stage == 1) { // восстанавление
			$sql = 'select count(*) from cl1_clan where reward_status = 2 limit 1';
			$sth = \db::$pdo->query($sql);
			if ($sth->fetchColumn() > 0) { // если награда уже начислялась
				return false;
			}
			// получить только те кланы у которых счет больше 1
			$sql = 'select * from cl1_clan where score > 0';
			$sth = \db::$pdo->query($sql);
			$clans = $sth->fetchAll(\PDO::FETCH_OBJ);
			foreach ($clans as $clan) {
				$reward = self::reward_generate($clan->score);
				self::clan_reward_update($clan->id_clan, $reward); // начислить награду (во временный столбец)
				self::clan_reward_status_update($clan->id_clan, 2); // поставить отметку что награда начислена
				
				// записать в журнал
				$bucks_frmt = number_format($reward['bucks']);
				$metal_frmt = number_format($reward['metal']);
				$mutagen_frmt = number_format($reward['mutagen']);
				$text = "
					За Битву Шредера клан получает:
					<span class='bucks-color'>$bucks_frmt баксов</span>,
					<span class='metal-color'>$metal_frmt металлолома</span>,
					<span class='mutagen-color'>$mutagen_frmt мутагена</span>
				";
				\clan_journal::record_add($text, $clan->id_clan);
			}
		}
		elseif ($stage == 2) { // прохождение
			$sql = 'select count(*) from cl1_clan where reward_status = 1 limit 1';
			$sth = \db::$pdo->query($sql);
			if ($sth->fetchColumn() < 1) {
				return false;
			}
			// обнулить отметку о начислении награды (чтоб было возможно начислить новую, когда окончится сражение)
			$sql = 'update cl1_clan set reward_status = 1 where reward_status != 1 limit 1';
			$sth = \db::$pdo->prepare($sql);
			$sth->execute();
		}
	}
}































class cl1_model
{
	public function cl1()
	{
		$user_id = ID;
		$user_clan_id = clan::get_id_clan_by_id_user($user_id);
		$user_clan_name = clan::get_clan_name_by_id_clan($user_clan_id);
		$stage = cl1::loc_get_stage();
		$user_in_clan = clan::is_user_clan($user_id);
		$user_score = cl1::get_user_score($user_id, $user_clan_id);
		$user_score_last = cl1::get_user_score_last($user_id);
		$user_score_percent = [
			'in_clan' => round(cl1::get_user_score_percent_in_clan($user_id, $user_clan_id), 1),
			'in_overall' => round(cl1::get_user_score_percent_in_overall($user_id), 1)
		];
		$clan_score = cl1::get_score_clan($user_clan_id);
		$clan_score_percent = round(cl1::get_score_clan_percent($user_clan_id), 1);
		$clan_reward = cl1::get_clan_reward($user_clan_id);
		$clan_reward_last = cl1::reward_generate($clan_score);
		$user_battles = cl1::get_user_battles($user_id);
		$user_battles_recovery_timestamp = cl1::get_user_battles_recovery_timestamp($user_id);
		$user_battles_recovery_seconds_left = cl1::get_user_battles_recovery_seconds_left($user_id);
		$loc_time_recovery_left = cl1::loc_time_recovery_left();
		$loc_time_passing_left = cl1::loc_time_passing_left();
		$part_overall = cl1::get_part_overall(5, false, true);
		$part_clan = cl1::get_part_clan(clan::get_id_clan_by_id_user($user_id), 5, false, true);
		foreach ($part_overall as $key => $val) {
			$part_overall[$key]['name'] = get_username_by_id($val[0]);
			$part_overall[$key]['score_percent'] = round(cl1::get_user_score_percent_in_overall($val[0]), 1);
		}
		foreach ($part_clan as $key => $val) {
			$part_clan[$key]['name'] = get_username_by_id($val[0]);
			$part_clan[$key]['score_percent'] = round(cl1::get_user_score_percent_in_clan($val[0], $user_clan_id), 1);
		}
		$_clans = cl1::get_clans(5, false, true);
		for ($i = 0; $i < 4; $i++) {
			if (isset($_clans[$i])) {
				$clans[$i] = $_clans[$i];
				$clans[$i]['clan_name'] = clan::get_clan_name_by_id_clan($_clans[$i]['id_clan']);
				$clans[$i]['score'] = round(cl1::get_score_clan($_clans[$i]['id_clan']), 1);
				$clans[$i]['score_percent'] = round(cl1::get_score_clan_percent($_clans[$i]['id_clan']), 1);
			}
			else {
				$clans[$i] = null;
			}
		}
		if (count($clans) < 5) {
			$clans[] = null;
		}
		
		$is_user_get_reward_privilege = false;
		if ($user_in_clan) {
			$is_user_get_reward_privilege = cl1::is_user_get_reward_privilege($user_id);
		}
		return [
			'stage' => $stage,
			'user_in_clan' => $user_in_clan,
			'user_clan_id' => $user_clan_id,
			'user_clan_name' => $user_clan_name,
			'user_score' => $user_score,
			'user_score_last' => $user_score_last,
			'user_score_percent' => $user_score_percent,
			'clan_score' => $clan_score,
			'clan_score_percent' => $clan_score_percent,
			'clan_reward' => $clan_reward,
			'clan_reward_last' => $clan_reward_last,
			'user_battles' => $user_battles,
			'user_battles_recovery_timestamp' => $user_battles_recovery_timestamp,
			'user_battles_recovery_seconds_left' => $user_battles_recovery_seconds_left,
			'loc_time_recovery_left' => $loc_time_recovery_left,
			'loc_time_passing_left' => $loc_time_passing_left,
			'part_overall' => $part_overall,
			'part_clan' => $part_clan,
			'clans' => $clans,
			'is_user_get_reward_privilege' => $is_user_get_reward_privilege
		];
	}
	
	public function cl1_statsc($statsc_type, $user_id = false, $clan_id = null)
	{
		user_id($user_id);
		$user_clan_id = \clan::get_id_clan_by_id_user($user_id);
		$clan_name = '';
		$pag = new pagination;
		if ($statsc_type == 'clans') {
			$clans_num = cl1::clans_count();
			$limit = $pag->start($clans_num, 10);
			$statsc = cl1::get_clans($limit['limit'], $limit['start'], true);
			foreach ($statsc as $key => $clan) {
				$statsc[$key]['id_clan'] = $clan['id_clan'];
				$statsc[$key]['name'] = clan::get_clan_name_by_id_clan($clan['id_clan']);
				$statsc[$key]['score'] = cl1::get_score_clan($clan['id_clan']);
				$statsc[$key]['score_percent'] = round(cl1::get_score_clan_percent($clan['id_clan']), 1);
			}
		}
		elseif ($statsc_type == 'users') {
			$users_num = cl1::part_count();
			$limit = $pag->start($users_num, 10);
			$statsc = cl1::get_part_overall($limit['limit'], $limit['start'], true);
			foreach ($statsc as $key => $val) {
				$statsc[$key]['id_user'] = $val[0];
				$statsc[$key]['name'] = get_username_by_id($val[0]);
				$statsc[$key]['score'] = cl1::get_user_score($val[0]);
				$statsc[$key]['score_percent'] = round(cl1::get_user_score_percent_in_overall($val[0]), 1);
			}
		}
		elseif ($statsc_type == 'clan_users') {
			$clan_users_num = cl1::clan_part_count($clan_id);
			$limit = $pag->start($clan_users_num, 10);
			$statsc = cl1::get_part_clan($clan_id, $limit['limit'], $limit['start'], true);
			foreach ($statsc as $key => $val) {
				$statsc[$key]['id_user'] = $val[0];
				$statsc[$key]['name'] = get_username_by_id($val[0]);
				$statsc[$key]['score'] = cl1::get_user_score_in_clan($val[0], $clan_id);
				$statsc[$key]['score_percent'] = round(cl1::get_user_score_percent_in_clan($val[0], $clan_id), 1);
			}
			$clan_name = clan::get_clan_name_by_id_clan($clan_id);
		}
		return [
			'statsc_type' => $statsc_type,
			'pag' => $pag,
			'user_id' => $user_id,
			'user_clan_id' => $user_clan_id,
			'statsc' => $statsc,
			'clan_name' => $clan_name,
			'clan_id' => $clan_id
		];
	}
}