<?php

class clan
{
	const CLAN_CREATE_MUTAGEN_PRICE = 100; // цена создания клана в мутагене
	const CLAN_CAPACITY_DEFAULT = 25; // базовая вместимость клана
	
	public static function rank_rus($rank = false)
	{
		$rus = [
			'leader' => 'Лидер',
			'deputy' => 'Заместитель',
			'assistant' => 'Помощник',
			'his' => 'Свояк',
			'recruit' => 'Новобранец'
		];
		if ($rank) {
			return $rus[$rank];
		}
		return $rus;
	}
	
	public static function rank_whom_rus($rank = false)
	{
		$rus = [
			'leader' => 'Лидера',
			'deputy' => 'Заместителя',
			'assistant' => 'Помощника',
			'his' => 'Свояка',
			'recruit' => 'Новобранца'
		];
		if ($rank) {
			return $rus[$rank];
		}
		return $rus;
	}
	
	// существует ли клан с id
	public static function is_clan($id_clan)
	{
		$sql = 'select count(*) from clan where id = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_clan]);
		if ($sth->fetch(PDO::FETCH_NUM)[0] > 0) {
			return true;
		}
		return false;
	}
	
	// состоит ли игрок в клане
	// если указан параметр id_clan, то проверяет состоит ли игрок в клане с этим id
	public static function is_user_clan($id_user, $id_clan = false)
	{
		$w2 = '';
		if ($id_clan) {
			$w2 = 'and id_clan = ?';
		}
		$sql = "select count(*) from clan_part where id_user = ? $w2 limit 1";
		$sth = db::$pdo->prepare($sql);
		if ($id_clan) {
			$sth->execute([$id_user, $id_clan]);
		}
		else {
			$sth->execute([$id_user]);
		}
		if ($sth->fetch(PDO::FETCH_NUM)[0] > 0) {
			return true;
		}
		return false;
	}
	
	// узнать id звания по званию
	public static function get_rank_id_by_rank_name($rank_name)
	{
		return array_flip(package('clan-privilege'))[$rank_name];
	}
	
	// узнать name звания по id звания
	public static function get_rank_name_by_rank_id($rank_id)
	{
		return package('clan-privilege')[$rank_id];
	}
	
	// узнать id-клана игрока
	public static function get_id_clan_by_id_user($id_user)
	{
		$sql = 'select id_clan from clan_part where id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_user]);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// узнать id последнего клана в котором находился игрок
	public static function get_user_id_clan_last($id_user)
	{
		$sql = 'select id_clan_last from users where id = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_user]);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// достать id игроков по id клана
	public static function get_clan_part_id_by_id_clan($id_clan, $limit = false, $start = false)
	{
		$l = '';
		if (is_numeric($start)) {
			$l = "$start,";
		}
		if (is_numeric($limit)) {
			$l = "limit $l$limit";
		}
		$sql = "select id_user from clan_part where id_clan = ? order by rank desc, exp desc $l";
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_clan]);
		return $sth->fetchAll(PDO::FETCH_NUM);
	}
	
	// узнать звание игрока в клане
	public static function get_user_clan_rank($id_user)
	{
		$sql = 'select rank from clan_part where id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_user]);
		return package('clan-privilege')[$sth->fetch(PDO::FETCH_NUM)[0]];
	}
	
	// узнать вместимость клана по id
	public static function get_clan_capacity_by_id_clan($id_clan)
	{
		$sql = 'select capacity from clan where id = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_clan]);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// узнать текущее количество игроков в клане
	public static function get_clan_part_num_by_id_clan($id_clan)
	{
		$sql = 'select count(*) from clan_part where id_clan = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_clan]);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// узнать название клана по его id
	public static function get_clan_name_by_id_clan($id_clan)
	{
		$sql = 'select name from clan where id = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_clan]);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// узнать описание клана по его id
	public static function get_clan_description_by_id_clan($id_clan)
	{
		$sql = 'select description from clan where id = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_clan]);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// может ли игрок менять описание своего клана
	public static function avail_user_clan_description_exchange($id_user)
	{
		if ( ! self::is_user_clan($id_user)) {
			return false;
		}
		$privilege = [
			'leader' => true,
			'deputy' => true,
			'assistant' => false,
			'his' => false,
			'recruit' => false
		];
		$rank = clan::get_user_clan_rank($id_user);
		if ( ! $privilege[$rank]) {
			return false;
		} return true;
	}
	
	// сменить описание клана
	public static function set_clan_description($id_clan, $description)
	{
		$sql = 'update clan set description = ? where id = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		return $sth->execute([$description, $id_clan]);
	}
	
	// узнать сколько опыта игрок принес в клан
	public static function get_clan_part_exp_by_id_user($id_user)
	{
		$sql = 'select exp from clan_part where id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_user]);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// узнать сколько баксов игрок принес в клан
	public static function get_clan_part_bucks_by_id_user($id_user)
	{
		$sql = 'select bucks from clan_part where id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_user]);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// узнать сколько баксов игрок принес в клан
	public static function get_clan_part_mutagen_by_id_user($id_user)
	{
		$sql = 'select mutagen from clan_part where id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_user]);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// узнать сколько баксов игрок принес в клан
	public static function get_clan_part_pancers_by_id_user($id_user)
	{
		$sql = 'select pancers from clan_part where id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_user]);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// узнать сколько баксов игрок принес в клан
	public static function get_clan_part_metal_by_id_user($id_user)
	{
		$sql = 'select metal from clan_part where id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_user]);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// получить временную метку, когда игрок присоеденился к клану
	public static function get_clan_part_join_timestamp($id_user)
	{
		$sql = 'select join_timestamp from clan_part where id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_user]);
		return $sth->fetchColumn();
	}
	
	// получить прошедшие секунды с момента вступления игроком в клан
	public static function get_clan_part_join_seconds_gone($id_user)
	{
		return time() - self::get_clan_part_join_timestamp($id_user);
	}
	
	public static function get_clan_created_timestamp($id_clan)
	{
		$sql = 'select created_timestamp from clan where id = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_clan]);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}	
	
	public static function set_clan_created_timestamp($id_clan)
	{
		$time = time();
		$sql = 'update clan set created_timestamp = ? where id = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$time, $id_clan]);
	}
	
	public static function get_clan_week_reset_timestamp($id_clan)
	{
		$sql = 'select week_reset_timestamp from clan where id = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_clan]);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	public static function set_clan_week_reset_timestamp($id_clan)
	{
		$time = time();
		$sql = 'update clan set week_reset_timestamp = ? where id = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$time, $id_clan]);
	}
	
	
	// записать опыт
	public static function clan_part_exp_set($id_user, $exp)
	{
		$sql = 'update clan_part set exp = exp + ?, exp_week = exp_week + ? where id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$exp, $exp, $id_user]);
	}
	
	// записать баксы
	public static function clan_part_bucks_set($id_user, $bucks)
	{
		$sql = 'update clan_part set bucks = bucks + ?, bucks_week = bucks_week + ? where id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$bucks, $bucks, $id_user]);
	}
	
	// записать мутаген
	public static function clan_part_mutagen_set($id_user, $mutagen)
	{
		$sql = 'update clan_part set mutagen = mutagen + ?, mutagen_week = mutagen_week + ? where id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$mutagen, $mutagen, $id_user]);
	}
	
	// записать панцири
	public static function clan_part_pancers_set($id_user, $pancers)
	{
		$sql = 'update clan_part set pancers = pancers + ?, pancers_week = pancers_week + ? where id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$pancers, $pancers, $id_user]);
	}
	
	// записать метал
	public static function clan_part_metal_set($id_user, $metal)
	{
		$sql = 'update clan_part set metal = metal + ?, metal_week = metal_week + ? where id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$metal, $metal, $id_user]);
	}
	
	// есть ли место в клане
	public static function is_clan_seats($id_clan)
	{
		$capct = self::get_clan_capacity_by_id_clan($id_clan);
		$cpn = self::get_clan_part_num_by_id_clan($id_clan);
		if ($cpn < $capct) { // место есть
			return true;
		}
		return false;
	}
	
	// имеет ли игрок с id право приглашать
	public static function is_user_invite_privilege($id_user)
	{
		$rank = self::get_user_clan_rank($id_user);
		$pinv = package('clan-privilege-invite')[$rank];
		return $pinv;
	}
	
	// достаточно ли привилегий у id_user_1 чтобы выгнать id_user_2
	public static function is_user_expel_privilege($id_user_1, $id_user_2)
	{
		$user_1_rank = self::get_user_clan_rank($id_user_1);
		$user_2_rank = self::get_user_clan_rank($id_user_2);
		$cpe = package('clan-privilege-expel');
		if (in_array($user_2_rank, $cpe[$user_1_rank])) {
			return true;
		}
		return false;
	}
	
	// достаточно ли привилегий у id_user_1 чтобы повысить id_user_2
	public static function is_user_increase_privilege($id_user_1, $id_user_2)
	{
		$user_1_rank = self::get_user_clan_rank($id_user_1);
		$user_2_rank = self::get_user_clan_rank($id_user_2);
		$pinc = package('clan-privilege-increase');
		if (in_array($user_2_rank, $pinc[$user_1_rank])) {
			return true;
		}
		return false;
	}
	
	// достаточно ли привилегий у id_user_1 чтобы понизить id_user_2
	public static function is_user_lower_privilege($id_user_1, $id_user_2)
	{
		$user_1_rank = self::get_user_clan_rank($id_user_1);
		$user_2_rank = self::get_user_clan_rank($id_user_2);
		$plow = package('clan-privilege-lower');
		if (in_array($user_2_rank, $plow[$user_1_rank])) {
			return true;
		}
		return false;
	}
	
	
	// проверить находятся ли два игрока в одном и том же клане
	public static function is_two_users_clan_same($id_user_1, $id_user_2)
	{
		if (self::get_id_clan_by_id_user($id_user_1) == self::get_id_clan_by_id_user($id_user_2)) {
			return true;
		}
		return false;
	}
	
	// есть ли приглашение у игрока в определенный клан
	public static function is_user_invite_to_clan($id_user, $id_clan)
	{
		$sql = 'select count(*) from clan_invite where id_user = ? and id_clan = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_user, $id_clan]);
		if ($sth->fetch(PDO::FETCH_NUM)[0] > 0) {
			return true;
		}
		return false;
	}
	
	// сколько у игрока есть приглашений в кланы
	public static function user_clan_invite_num($id_user)
	{
		$sql = 'select count(*) from clan_invite where id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_user]);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// может ли $id_user_1 пригласить $id_user_2
	public static function is_user_invite_user($id_user_1, $id_user_2)
	{
		if ( ! self::is_user_clan($id_user_1) || self::is_user_clan($id_user_2)) { // если user_1 не в клане или user_2 в клане
			return false;
		}
		if ( ! self::is_user_invite_privilege($id_user_1)) { // если у user_1 недостаточно привилегий
			return false;
		}
		$id_clan = self::get_id_clan_by_id_user($id_user_1); // id клана user_1
		if ( ! self::is_clan_seats($id_clan)) { // если в клане нету мест
			return false;
		}
		if (self::is_user_invite_to_clan($id_user_2, $id_clan)) { // у user_2 уже есть приглашение в этот клан
			return false;
		}
		return true;
	}
	
	// может ли $id_user_1 исключить $id_user_2
	public static function is_user_expel_user($id_user_1, $id_user_2)
	{
		if ( ! self::is_user_clan($id_user_1) || ! self::is_user_clan($id_user_2)) { // если user_1 не в клане или user_2 не в клане
			return false;
		}
		if ( ! self::is_two_users_clan_same($id_user_1, $id_user_2)) { // если user_1 и user_2 в разных кланах
			return false;
		}
		if ( ! self::is_user_expel_privilege($id_user_1, $id_user_2)) { // если у user_1 недостаточно привилегий
			return false;
		}
		return true;
	}
	
	// может ли $id_user_1 повысить $id_user_2
	public static function is_user_increase_user($id_user_1, $id_user_2)
	{
		if ( ! self::is_user_clan($id_user_1) || ! self::is_user_clan($id_user_2)) { // если user_1 не в клане или user_2 не в клане
			return false;
		}
		if ( ! self::is_two_users_clan_same($id_user_1, $id_user_2)) { // если user_1 и user_2 в разных кланах
			return false;
		}
		if ( ! self::is_user_increase_privilege($id_user_1, $id_user_2)) { // если у user_1 недостаточно привилегий
			return false;
		}
		return true;
	}
	
	// может ли $id_user_1 понизить $id_user_2
	public static function is_user_lower_user($id_user_1, $id_user_2)
	{
		if ( ! self::is_user_clan($id_user_1) || ! self::is_user_clan($id_user_2)) { // если user_1 не в клане или user_2 не в клане
			return false;
		}
		if ( ! self::is_two_users_clan_same($id_user_1, $id_user_2)) { // если user_1 и user_2 в разных кланах
			return false;
		}
		if ( ! self::is_user_lower_privilege($id_user_1, $id_user_2)) { // если у user_1 недостаточно привилегий
			return false;
		}
		return true;
	}
	
	// может ли игрок вступить в клан
	public static function is_user_clan_go($id_user, $id_clan)
	{
		if ( ! self::is_user_invite_to_clan($id_user, $id_clan)) { // если у игрока нету приглашения
			return false;
		}
		if ( ! self::is_clan_seats($id_clan)) { // если в клане нету мест
			return false;
		}
		if (self::is_user_clan($id_user)) { // если игрок уже в клане
			return false;
		}
		return true;
	}
	
	// может ли игрок покинуть клан
	public static function is_user_clan_exit($id_user)
	{
		if ( ! clan::is_user_clan($id_user)) { // если игрок не в клане
			return false;
		}
		return true;
	}

	
	// получить id всех кланов которые приглашают игрока в свои ряды
	public static function get_id_clans_invite_by_id_user($id_user, $limit = false, $start = false)
	{
		$l = '';
		if ($start !== false) {
			$l = "limit $start,$limit";
		}
		elseif ($limit !== false) {
			$l = "limit $limit";
		}
		$sql = "select id_clan from clan_invite where id_user = ? $l";
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_user]);
		return $sth->fetchAll(PDO::FETCH_NUM);
	}
	
	
	
	
	
	
	
	// создать новый клан
	public static function clan_create($id_leader, $name)
	{
		$time = time();
		$sql = 'insert into clan set name = ?, capacity = ?, created_timestamp = ?, week_reset_timestamp = ?';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$name, self::CLAN_CAPACITY_DEFAULT, $time, $time]);
		$clan_last_id = db::$pdo->lastinsertid();
		return $clan_last_id;
	}
	
	// послать приглашение игроку id_user в клан id_clan
	public static function clan_invite_send($id_user, $id_clan)
	{
		$sql = 'insert into clan_invite set id_user = ?, id_clan = ?';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_user, $id_clan]);
	}
	
	// установить id клана в id_clan_last (id последнего клана в котором находился игрок)
	public static function set_user_id_clan_last($id_user, $id_clan)
	{
		$sql = 'update users set id_clan_last = ? where id = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_clan, $id_user]);
	}
	
	// завести игрока в клан
	public static function user_clan_go($id_user, $id_clan)
	{
		$sql = 'insert into clan_part set id_user = ?, id_clan = ?, rank = ?, join_timestamp = ?';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_user, $id_clan, '1', time()]);
	}
	
	// повысить игрока на один ранг
	// если параметр rank задан, то повышает до определенного звания
	public static function user_clan_up($id_user, $rank = false)
	{
		$r = 'rank = rank + 1';
		if ($rank) {
			$r = "rank = ?";
		}
		$sql = "update clan_part set $r where id_user = ? limit 1";
		$sth = db::$pdo->prepare($sql);
		if ($rank) {
			$sth->execute([$rank, $id_user]);
		}
		else {
			$sth->execute([$id_user]);
		}
	}
	
	// понизить игрока
	public static function user_clan_down($id_user)
	{
		$sql = 'update clan_part set rank = rank - 1 where id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_user]);
	}
	
	// вывести игрока из клана
	public static function user_clan_exit($id_user)
	{
		$sql = 'delete from clan_part where id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_user]);
	}
	
	// удалить приглашение для игрока в определенный клан
	public static function clan_invite_user_delete($id_user, $id_clan)
	{
		$sql = 'delete from clan_invite where id_user = ? and id_clan = ?';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_user, $id_clan]);
	}
	
	// включить игрока в клан со всеми вытекающими
	public static function user_clan_enable($id_user, $id_clan)
	{
		$pb = clan_building::get_residence_param_bonus_all_by_id_clan($id_clan); // параметры от резиденции
		uc::update_param_all($id_user, $pb); // изменить параметры
		self::user_clan_go($id_user, $id_clan); // завести
		self::clan_invite_user_delete($id_user, $id_clan); // удалить это приглашение
		\clan\app::delete_apps($id_user); // удалить заявки в другие кланы этим игроком
		self::set_user_id_clan_last($id_user, $id_clan);
	}
	
	// выключить игрока из клана со всеми вытекающими
	public static function user_clan_disable($id_user)
	{
		$id_clan = self::get_id_clan_by_id_user($id_user);
		$pb = clan_building::get_residence_param_bonus_all_by_id_clan($id_clan); // параметры от резиденции
		uc::update_param_all($id_user, $pb, true);
		self::user_clan_exit($id_user);
	}
	
	// создать запись для клана с клановой локацией-1
	public static function cl1_record_create($id_clan)
	{
		$sql = 'insert into cl1_clan set id_clan = ?';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_clan]);
	}
	
	// контроллер создания клана
	public static function clan_create_ctrl($id_leader, $name)
	{
		if (self::is_user_clan($id_leader)) { // игрок уже состоит в клане
			return false;
		}
		$mutagen = get_mutagen_by_id($id_leader);
		if ($mutagen < self::CLAN_CREATE_MUTAGEN_PRICE) { // если средств недостаточно
			notf::set_notf('Недостаточно средств', 'error');
			return false;
		}
		$name_check = filter::clan_name_check($name);
		if ($name_check == 2) {
			notf::set_notf('Название не должно быть пустым', 'error');
			return;
		}
		elseif ($name_check == 4) {
			notf::set_notf('Название не соответствующей длины', 'error');
			return;
		}
		elseif ($name_check == 8) {
			notf::set_notf('В названии присутствуют запрещенные символы', 'error');
			return;
		}
		elseif ($name_check == 16) {
			notf::set_notf('Название должно содержать хотя бы одну букву или цифру', 'error');
			return;
		}
		elseif ($name_check != 2020) {
			return;
		}
		
		$id_clan = self::clan_create($id_leader, $name); // создать
		self::user_clan_enable($id_leader, $id_clan); // завести в клан создавшего
		self::user_clan_up($id_leader, self::get_rank_id_by_rank_name('leader')); // повысить до лидера
		set_mutagen(-self::CLAN_CREATE_MUTAGEN_PRICE, $id_leader); // снять мутаген
		clan_building::building_record_create($id_clan); // создать записи с постройками клана
		self::cl1_record_create($id_clan); // создать строку битвы шредера
		// удалить все заявки
		
		$leader_login = user::login_get($id_leader);
		$leader_link = subord_url('profile', "id_user=$id_leader");
		$text = "<a href='$leader_link'>$leader_login</a> создал(/а) клан";
		clan_journal::record_add($text, $id_clan);
		return true;
	}
	
	// контроллер отправки приглашения игроку
	public static function clan_invite_send_ctrl($id_user_1, $id_user_2)
	{
		if ( ! self::is_user_invite_user($id_user_1, $id_user_2)) { // если user_1 не может отправить приглашение user_2
			return false;
		}
		$id_clan = self::get_id_clan_by_id_user($id_user_1); // id клана user_1
		self::clan_invite_send($id_user_2, $id_clan);
		dyn_set_to_zero_clan_invite_new($id_user_2, 1);
		
		$id_user_1_login = user::login_get($id_user_1);
		$id_user_2_login = user::login_get($id_user_2);
		$user_1_link = subord_url('profile', "id_user=$id_user_1");
		$user_2_link = subord_url('profile', "id_user=$id_user_2");
		$text = "<a href='$user_1_link'>$id_user_1_login</a> отправил(/а) приглашение <a href='$user_2_link'>$id_user_2_login</a>";
		clan_journal::record_add($text, $id_clan);
		notf::set_notf('Приглашение отправлено.');
	}
	
	// контроллер исключения игрока из клана
	public static function clan_expel_user_ctrl($id_user_1, $id_user_2)
	{
		if ( ! self::is_user_expel_user($id_user_1, $id_user_2)) { // user_1 не может исключить user_2 по каким-то причинам
			return false;
		}
		self::user_clan_disable($id_user_2);
		$id_user_1_login = user::login_get($id_user_1);
		$id_user_2_login = user::login_get($id_user_2);
		$user_1_link = subord_url('profile', "id_user=$id_user_1");
		$user_2_link = subord_url('profile', "id_user=$id_user_2");
		$text = "<a href='$user_1_link'>$id_user_1_login</a> исключил(/а) <a href='$user_2_link'>$id_user_2_login</a>";
		$id_clan = self::get_id_clan_by_id_user($id_user_1);
		clan_journal::record_add($text, $id_clan);
		$clan_link = subord_url('clan', "id_clan=$id_clan");
		$clan_name = self::get_clan_name_by_id_clan($id_clan);
		$title = "Клан: «<a href='$clan_link'>{$clan_name}</a>»";
		$text = "<a href='$user_1_link'>$id_user_1_login</a> исключил(/а) вас из клана";
		journal::record_add(['title' => $title, 'text' => $text], $id_user_2);
	}
	
	// контроллер повышения игрока
	public static function clan_increase_user_ctrl($id_user_1, $id_user_2)
	{
		if ( ! self::is_user_increase_user($id_user_1, $id_user_2)) { // user_1 не может повысить user_2 по каким-то причинам
			return false;
		}
		self::user_clan_up($id_user_2);
		$user_2_rank = self::get_user_clan_rank($id_user_2);
		$id_user_1_login = user::login_get($id_user_1);
		$id_user_2_login = user::login_get($id_user_2);
		$link_user_1 = subord_url('profile', "id_user=$id_user_1");
		$link_user_2 = subord_url('profile', "id_user=$id_user_2");
		$text = "<a href='$link_user_1'>$id_user_1_login</a> повысил(/а) <a href='$link_user_2'>$id_user_2_login</a> до " . self::rank_whom_rus($user_2_rank);
		$id_clan = self::get_id_clan_by_id_user($id_user_1);
		clan_journal::record_add($text, $id_clan);
		$clan_link = subord_url('clan', "id_clan=$id_clan");
		$clan_name = self::get_clan_name_by_id_clan($id_clan);
		$title = "Клан: «<a href='$clan_link'>{$clan_name}</a>»";
		$text = "<a href='$link_user_1'>$id_user_1_login</a> повысил(/а) вас до " . self::rank_whom_rus($user_2_rank);
		journal::record_add(['title' => $title, 'text' => $text], $id_user_2);
	}
	
	// контроллер понижения игрока
	public static function clan_lower_user_ctrl($id_user_1, $id_user_2)
	{
		if ( ! self::is_user_lower_user($id_user_1, $id_user_2)) { // user_1 не может понизить user_2 по каким-то причинам
			return false;
		}
		self::user_clan_down($id_user_2);
		$user_2_rank = self::get_user_clan_rank($id_user_2);
		$id_user_1_login = user::login_get($id_user_1);
		$id_user_2_login = user::login_get($id_user_2);
		$link_user_1 = subord_url('profile', "id_user=$id_user_1");
		$link_user_2 = subord_url('profile', "id_user=$id_user_2");
		$text = "<a href='$link_user_1'>$id_user_1_login</a> понизил(/а) <a href='$link_user_2'>$id_user_2_login</a> до " . self::rank_whom_rus($user_2_rank);
		$id_clan = self::get_id_clan_by_id_user($id_user_1);
		clan_journal::record_add($text, $id_clan);
		$clan_link = subord_url('clan', "id_clan=$id_clan");
		$clan_name = self::get_clan_name_by_id_clan($id_clan);
		$title = "Клан: «<a href='$clan_link'>{$clan_name}</a>»";
		$text = "<a href='$link_user_1'>$id_user_1_login</a> понизил(/а) вас до ". self::rank_whom_rus($user_2_rank);
		journal::record_add(['title' => $title, 'text' => $text], $id_user_2);
	}
	
	// контроллер захода в клан игроком
	public static function user_clan_go_ctrl($id_user, $id_clan)
	{
		if ( ! self::is_user_clan_go($id_user, $id_clan)) { // игрок по каким-то причинам не может зайти в клан
			return false;
		}
		
		$config = require ROOT . 'project/internal/config/main.config.php';
		$need_level = $config['user_enter_clan_need_level'];
		$user_level = \user::level_get($id_user);
		if ($user_level < $need_level) {
			\notf::set_notf("Для вступления в клан нужен $need_level-й уровень.", 'error');
			return false;
		}
		
		self::user_clan_enable($id_user, $id_clan);
		$user_login = user::login_get($id_user);
		$user_link = subord_url('profile', "id_user=$id_user");
		$text = "<a href='$user_link'>$user_login</a> вступил(/а) в клан";
		clan_journal::record_add($text, $id_clan);
		notf::set_notf('Вы вступили в клан.', 'default');
	}
	
	// контроллер выхода из клана игроком
	public static function user_clan_exit_ctrl($id_user)
	{
		if ( ! self::is_user_clan_exit($id_user)) { // игрок по каким-то причинам не может выйти из клана
			return false;
		}
		
		$is_leader = (self::get_user_clan_rank($id_user) == 'leader') ? true : false;
		
		$id_clan = self::get_id_clan_by_id_user($id_user);
		self::user_clan_disable($id_user);
		$user_login = user::login_get($id_user);
		$user_link = subord_url('profile', "id_user=$id_user");
		$text = "<a href='$user_link'>$user_login</a> покинул(/а) клан";
		clan_journal::record_add($text, $id_clan);
		\notf::set_notf('Вы покинули клан.', 'default');
		
		/* если клан покидает лидер */
		if ($is_leader) {
			if (self::get_clan_part_num_by_id_clan($id_clan) < 1) { // кроме лидера, который покидает клан, в клане никого нет
				return false;
			}
			$sql = 'select count(*) from clan_part where id_clan = ? and rank = 4 limit 1';
			$sth = db::$pdo->prepare($sql);
			$sth->execute([$id_clan]);
			if ($sth->fetchColumn() > 0) { // если заместители есть
				// достать id заместителя с самым большим опытом
				$sql = 'select id_user from clan_part where id_clan = ? and rank = 4 order by exp desc limit 1';
				$sth = db::$pdo->prepare($sql);
				$sth->execute([$id_clan]);
				$new_leader_id = $sth->fetchColumn();
				self::user_clan_up($new_leader_id, 5);
				return;
			}
			
			$sql = 'select count(*) from clan_part where id_clan = ? and rank = 3 limit 1';
			$sth = db::$pdo->prepare($sql);
			$sth->execute([$id_clan]);
			if ($sth->fetchColumn() > 0) { // если помощники есть
				// достать id помощника с самым большим опытом
				$sql = 'select id_user from clan_part where id_clan = ? and rank = 3 order by exp desc limit 1';
				$sth = db::$pdo->prepare($sql);
				$sth->execute([$id_clan]);
				$new_leader_id = $sth->fetchColumn();
				self::user_clan_up($new_leader_id, 5);
				return;
			}
			
			// передать любому игроку с найбольшим опытом
			$sql = 'select count(*) from clan_part where id_clan = ? limit 1';
			$sth = db::$pdo->prepare($sql);
			$sth->execute([$id_clan]);
			if ($sth->fetchColumn() > 0) { // если помощники есть
				// достать id любого участника с самым большим опытом
				$sql = 'select id_user from clan_part where id_clan = ? order by exp desc limit 1';
				$sth = db::$pdo->prepare($sql);
				$sth->execute([$id_clan]);
				$new_leader_id = $sth->fetchColumn();
				self::user_clan_up($new_leader_id, 5);
				return;
			}
		}
	}
	
	// контроллер смены названия клана
	public static function clan_name_change($user_id, $clan_id, $name)
	{
		$name_check = filter::clan_name_check($name);
		if ($name_check == 2) {
			notf::set_notf('Название не должно быть пустым', 'error');
			return;
		}
		elseif ($name_check == 4) {
			notf::set_notf('Название не соответствующей длины', 'error');
			return;
		}
		elseif ($name_check == 8) {
			notf::set_notf('В названии присутствуют запрещенные символы', 'error');
			return;
		}
		elseif ($name_check == 16) {
			notf::set_notf('Название должно содержать хотя бы одну букву или цифру', 'error');
			return;
		}
		elseif ($name_check != 2020) {
			return;
		}
		
		$rank = self::get_user_clan_rank($user_id);
		if ($rank != 'leader') {
			return false;
		}
		
		$clan_mutagen = clan_coffers::get_mutagen($clan_id);
		$mutagen_need = game::CLAN_NAME_CHANGE_PRICE_MUTAGEN;
		if ($clan_mutagen < $mutagen_need) {
			return false;
		}
		
		$clan_name_old = self::get_clan_name_by_id_clan($clan_id);
		
		/*  */
		$name = htmlspecialchars($name);
		$sql = 'update clan set name = ? where id = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$name, $clan_id]);
		/*  */
		
		clan_coffers::mutagen_refill($clan_id, -$mutagen_need);
		
		$user_name = user::login_get($user_id);
		$user_link = subord_url('profile', "id_user=$user_id");
		$journal_text = "<a href='$user_link'>$user_name</a> сменил(/а) название клана с «{$clan_name_old}» на «{$name}». <span class='crimson'>-$mutagen_need мутагена</span>";
		clan_journal::record_add($journal_text, $clan_id);
		
		notf::set_notf('Название изменено.', 'default');
	}
}

















class clan_model
{
	public function clan($id_clan)
	{
		$this_user_id = ID;
		$user_clan_id = clan::get_id_clan_by_id_user(ID);
		$clan_name = clan::get_clan_name_by_id_clan($id_clan);
		$level = clan_level::level_get($id_clan);
		$level_max = levels_clan::max();
		$exp = clan_exp::exp_get($id_clan);
		$exp_need = ($level < $level_max) ? levels_clan::exp_need_get($level + 1) : 0;
		$clan_part_num = clan::get_clan_part_num_by_id_clan($id_clan);
		$clan_capacity = clan::get_clan_capacity_by_id_clan($id_clan);
		$part = clan::get_clan_part_id_by_id_clan($id_clan);
		$temp = [];
		foreach ($part as $p) {
			$temp[] = [
				'id_user' => $p[0],
				'username' => get_username_by_id($p[0]),
				'rank' => clan::rank_rus(clan::get_user_clan_rank($p[0])),
				'exp' => clan::get_clan_part_exp_by_id_user($p[0]),
				'bucks' => clan::get_clan_part_bucks_by_id_user($p[0]),
				'mutagen' => clan::get_clan_part_mutagen_by_id_user($p[0]),
				'pancers' => clan::get_clan_part_pancers_by_id_user($p[0]),
				'metal' => clan::get_clan_part_metal_by_id_user($p[0])
			];
		}
		$part = $temp;
		
		$resid_param_bonus = clan_building::get_residence_param_bonus_all_by_id_clan($id_clan);
		$clan_bonus = [
			'mutagen' => clan_building::get_scientific_complex_bonus_mutagen(db\clan_building::building_level_get($id_clan, 'scientific_complex')),
			'bucks' => clan_building::get_repository_bonus_bucks_percent(db\clan_building::building_level_get($id_clan, 'repository')),
			'metal' => clan_building::get_metal_warehouse_bonus_metal_percent(db\clan_building::building_level_get($id_clan, 'metal_warehouse')),
			'metal_warehouse' => clan_building::get_metal_warehouse_bonus_metal_percent(db\clan_building::building_level_get($id_clan, 'metal_warehouse')),
			'expr' => clan_building::get_training_hall_bonus_exp_percent(db\clan_building::building_level_get($id_clan, 'training_hall'))
		];
		
		$resid_level = db\clan_building::building_level_get($id_clan, 'residence');
		$coffers_bucks = clan_coffers::get_bucks($id_clan);
		$coffers_mutagen = clan_coffers::get_mutagen($id_clan);
		$coffers_pancers = clan_coffers::get_pancers($id_clan);
		$coffers_metal = clan_coffers::get_metal($id_clan);
		
		$app = new \clan\app_retain;
		$app->user_id = $this_user_id;
		$app->clan_id = $id_clan;
		$user_app_is_avail = $app->is_avail();
		$app_is_avail = \clan\app::is_app_avail($this_user_id);
		
		$app_new = \clan\app::count($id_clan);
		$chat_new = \chat::is_clan_chat_new($this_user_id);
		
		$description = \clan::get_clan_description_by_id_clan($id_clan);
		$created_timestamp = \clan::get_clan_created_timestamp($id_clan);
		
		return [
			'user_clan_id' => $user_clan_id,
			'clan_create_mutagen_price' => clan::CLAN_CREATE_MUTAGEN_PRICE,
			'clan_part' => $part,
			'clan_id' => $id_clan,
			'clan_name' => $clan_name,
			'clan_part_num' => $clan_part_num,
			'clan_capacity' => $clan_capacity,
			'resid_param_bonus' => $resid_param_bonus,
			'clan_bonus' => $clan_bonus,
			'resid_level' => $resid_level,
			'coffers_bucks' => $coffers_bucks,
			'coffers_mutagen' => $coffers_mutagen,
			'coffers_pancers' => $coffers_pancers,
			'coffers_metal' => $coffers_metal,
			'level' => $level,
			'level_max' => $level_max,
			'exp' => $exp,
			'exp_need' => $exp_need,
			'user_app_is_avail' => $user_app_is_avail,
			'app_is_avail' => $app_is_avail,
			'app_new' => $app_new,
			'chat_new' => $chat_new,
			'description' => $description,
			'created_timestamp' => $created_timestamp
		];
	}
	
	public function clan_not()
	{
		$user_id = ID;
		$clan_invite_num = clan::user_clan_invite_num(ID);
		$config = require ROOT . 'project/internal/config/main.config.php';
		$user_level = user::level_get($user_id);
		
		return [
			'clan_create_mutagen_price' => clan::CLAN_CREATE_MUTAGEN_PRICE,
			'clan_invite_num' => $clan_invite_num,
			'user_enter_clan_need_level' => $config['user_enter_clan_need_level'],
			'user_level' => $user_level
		];
	}
	
	public function clan_create()
	{
		return [
			'clan_create_mutagen_price' => clan::CLAN_CREATE_MUTAGEN_PRICE
		];
	}
	
	public function clan_invite()
	{
		$user_id = ID;
		
		// посчитать приглашения игрока
		$invite_num = \clan::user_clan_invite_num($user_id);
		
		$pag = new \pagination;
		$limit = $pag->start($invite_num, 5);
		
		$invite = clan::get_id_clans_invite_by_id_user($user_id, $limit['limit'], $limit['start']);
		
		$temp = [];
		foreach ($invite as $inv) {
			$temp[] = [
				'clan_id' => $inv[0],
				'clan_name' => clan::get_clan_name_by_id_clan($inv[0]),
				'clan_param' => clan_building::get_residence_param_bonus_all_by_id_clan($inv[0]),
				'clan_bonus' => [
					'scientific_complex' => clan_building::get_scientific_complex_bonus_mutagen(db\clan_building::building_level_get($inv[0], 'scientific_complex')),
					'repository' => clan_building::get_repository_bonus_bucks_percent(db\clan_building::building_level_get($inv[0], 'repository')),
					'metal_warehouse' => clan_building::get_metal_warehouse_bonus_metal_percent(db\clan_building::building_level_get($inv[0], 'metal_warehouse')),
					'metal_warehouse' => clan_building::get_metal_warehouse_bonus_metal_percent(db\clan_building::building_level_get($inv[0], 'metal_warehouse')),
					'training_hall' => clan_building::get_training_hall_bonus_exp_percent(db\clan_building::building_level_get($inv[0], 'training_hall'))
				]
			];
		}
		$invite = $temp;
		return [
			'pag' => $pag,
			'invite' => $invite
		];
	}
	
	public function clan_control($exit_confirm)
	{
		$user_id = ID;
		$clan_id = clan::get_id_clan_by_id_user($user_id);
		$rank = clan::get_user_clan_rank($user_id);
		$is_name_change = ($rank == 'leader') ? true : false;
		$name_change_price = game::CLAN_NAME_CHANGE_PRICE_MUTAGEN;
		
		$avail_user_clan_description_exchange = clan::avail_user_clan_description_exchange($user_id);
		$description = clan::get_clan_description_by_id_clan($clan_id);
		
		return [
			'exit_confirm' => $exit_confirm,
			'is_name_change' => $is_name_change,
			'name_change_price' => $name_change_price,
			'avail_user_clan_description_exchange' => $avail_user_clan_description_exchange,
			'description' => $description
		];
	}
	
	public function clan_coffers($id_clan)
	{
		$coffers_bucks = clan_coffers::get_bucks($id_clan);
		$coffers_mutagen = clan_coffers::get_mutagen($id_clan);
		$coffers_pancers = clan_coffers::get_pancers($id_clan);
		$coffers_metal = clan_coffers::get_metal($id_clan);
		
		return [
			'coffers_bucks' => $coffers_bucks,
			'coffers_mutagen' => $coffers_mutagen,
			'coffers_pancers' => $coffers_pancers,
			'coffers_metal' => $coffers_metal
		];
	}
	
	public function clan_building($id_clan)
	{
		$id_user = ID;
		$id_clan_user = clan::get_id_clan_by_id_user($id_user);
		$is_user_privilege_building_up = false;
		if ($id_clan_user) {
			$is_user_privilege_building_up = user_clan_building::is_user_privilege_building_increase($id_user);
		}
		
		$residence_level = db\clan_building::building_level_get($id_clan, 'residence');
		$scientific_complex_level = db\clan_building::building_level_get($id_clan, 'scientific_complex');
		$repository_level = db\clan_building::building_level_get($id_clan, 'repository');
		$metal_warehouse_level = db\clan_building::building_level_get($id_clan, 'metal_warehouse');
		$training_hall_level = db\clan_building::building_level_get($id_clan, 'training_hall');
		
		$residence_level_max = clan_building::get_building_level_max('residence');
		$residence_bonus_param = clan_building::get_param_bonus_by_residence_level($residence_level);
		$residence_price_up = clan_building::get_building_price_improv('residence', $residence_level);
		
		$scientific_complex_level_max = clan_building::get_building_level_max('scientific_complex');
		$scientific_complex_bonus_mutagen = clan_building::get_scientific_complex_bonus_mutagen($scientific_complex_level);
		$scientific_complex_price_up = clan_building::get_building_price_improv('scientific_complex', $scientific_complex_level);
		
		$repository_level_max = clan_building::get_building_level_max('repository');
		$repository_bonus_bucks_percent = clan_building::get_repository_bonus_bucks_percent($repository_level);
		$repository_price_up = clan_building::get_building_price_improv('repository', $repository_level);
		
		$metal_warehouse_level_max = clan_building::get_building_level_max('metal_warehouse');
		$metal_warehouse_bonus_metal_percent = clan_building::get_metal_warehouse_bonus_metal_percent($metal_warehouse_level);
		$metal_warehouse_price_up = clan_building::get_building_price_improv('metal_warehouse', $metal_warehouse_level);
		
		$training_hall_level_max = clan_building::get_building_level_max('training_hall');
		$training_hall_bonus_exp_percent = clan_building::get_training_hall_bonus_exp_percent($training_hall_level);
		$training_hall_price_up = clan_building::get_building_price_improv('training_hall', $training_hall_level);
		
		
		return [
			'id_clan' => $id_clan,
			'id_clan_user' => $id_clan_user,
			'is_user_privilege_building_up' => $is_user_privilege_building_up,
			'residence_level' => $residence_level,
			'residence_level_max' => $residence_level_max,
			'residence_bonus_param' => $residence_bonus_param,
			'residence_bonus_param_each_level' => clan_building::BUILDING_RESIDENCE_USER_PARAM_BONUS_EACH_LEVEL,
			'residence_price_up' => $residence_price_up,
			'scientific_complex_level' => $scientific_complex_level,
			'scientific_complex_level_max' => $scientific_complex_level_max,
			'scientific_complex_bonus_mutagen' => $scientific_complex_bonus_mutagen,
			'scientific_complex_price_up' => $scientific_complex_price_up,
			'repository_level' => $repository_level,
			'repository_level_max' => $repository_level_max,
			'repository_price_up' => $repository_price_up,
			'repository_bonus_bucks_percent' => $repository_bonus_bucks_percent,
			'metal_warehouse_level' => $metal_warehouse_level,
			'metal_warehouse_level_max' => $metal_warehouse_level_max,
			'metal_warehouse_price_up' => $metal_warehouse_price_up,
			'metal_warehouse_bonus_metal_percent' => $metal_warehouse_bonus_metal_percent,
			'training_hall_level' => $training_hall_level,
			'training_hall_level_max' => $training_hall_level_max,
			'training_hall_price_up' => $training_hall_price_up,
			'training_hall_bonus_exp_percent' => $training_hall_bonus_exp_percent
		];
	}
	
	public function clan_app()
	{
		$user_id = ID;
		$clan_id = \clan::get_id_clan_by_id_user($user_id);
		$pag = new pagination;
		$app_num = \clan\app::count($clan_id);
		$limit = $pag->start($app_num);
		$sql = "select ca.id_user, ca.app_timestamp, u.login, u.level, uc.health, uc.power, uc.agil, uc.intell from clan_app ca
		left join users u on u.id = ca.id_user
		left join users_combat uc on uc.id_user = ca.id_user
		where ca.id_clan = $clan_id order by ca.app_timestamp desc limit $limit[start], $limit[limit]";
		$sth = \db::$pdo->query($sql);
		$apps = $sth->fetchAll(\PDO::FETCH_ASSOC);
		return [
			'pag' => $pag,
			'apps' => $apps
		];
	}
}