<?php

namespace db
{
	class clan_building extends \_tbl_line
	{
		protected static $table = 'clan_building';
		
		// узнать уровень постройки (по строковому идентификатору)
		public static function building_level_get($clan_id, $building)
		{
			return self::get("{$building}_level", 'where id_clan = ?', [$clan_id]);
		}
		
		// увеличить уровень постройки
		public static function building_level_increase($clan_id, $building)
		{
			return self::set("{$building}_level = {$building}_level + 1", 'where id_clan = ?', [$clan_id]);
		}
	}
}

namespace
{
	class clan_building
	{
		const BUILDING_RESIDENCE_USER_PARAM_BONUS_EACH_LEVEL = 10; // прибавка к каждому параметру каждому игроку клана за каждый уровень резиденции
		
		// узнать сколько прибавит резиденция к параметрам за уровень резиденции
		public static function get_param_bonus_by_residence_level($res_level)
		{
			if ($res_level < 1) {
				return 0;
			} return $res_level * self::BUILDING_RESIDENCE_USER_PARAM_BONUS_EACH_LEVEL;
		}
		
		// получить массив с прибавками к параметрам за уровень резиденции
		public static function get_param_bonus_all_by_residence_level($res_level)
		{
			$bonus = self::get_param_bonus_by_residence_level($res_level);
			return [
				'health' => $bonus,
				'power' => $bonus,
				'agil' => $bonus,
				'intell' => $bonus
			];
		}
		
		// получить массив с прибавками к параметрам за уровень резиденции по id клана
		public static function get_residence_param_bonus_all_by_id_clan($id_clan)
		{
			$res_level = db\clan_building::building_level_get($id_clan, 'residence');
			return self::get_param_bonus_all_by_residence_level($res_level);
		}

		// узнать цены на улучшение постройки
		public static function get_building_price_improv($building, $level)
		{
			$sql = 'select price_bucks, price_metal, price_mutagen from _clan_buildings_levels where building = ? and level = ? limit 1';
			$sth = \db::$pdo->prepare($sql);
			$sth->execute([$building, $level]);
			$prices = $sth->fetchObject();
			return [
				'bucks' => $prices->price_bucks,
				'metal' => $prices->price_metal,
				'mutagen' => $prices->price_mutagen
			];
		}
		
		// узнать максимальный уровень постройки
		public static function get_building_level_max($building)
		{
			$sql = 'select max(level) from _clan_buildings_levels where building = ? limit 1';
			$sth = \db::$pdo->prepare($sql);
			$sth->execute([$building]);
			return $sth->fetchColumn();
		}
		
		// узнать бонус к мутагену от научного комплекса по уровню комплекса
		public static function get_scientific_complex_bonus_mutagen($level)
		{
			$sql = 'select bonus from _clan_buildings_levels where building = ? and level = ? limit 1';
			$sth = \db::$pdo->prepare($sql);
			$sth->execute(['scientific_complex', $level]);
			return $sth->fetchColumn();
		}
		
		// узнать бонус к баксам от хранилища
		public static function get_repository_bonus_bucks_percent($level)
		{
			$sql = 'select bonus from _clan_buildings_levels where building = ? and level = ? limit 1';
			$sth = \db::$pdo->prepare($sql);
			$sth->execute(['repository', $level]);
			return $sth->fetchColumn();
		}
		
		// узнать бонус к металу от металлобазы
		public static function get_metal_warehouse_bonus_metal_percent($level)
		{
			$sql = 'select bonus from _clan_buildings_levels where building = ? and level = ? limit 1';
			$sth = \db::$pdo->prepare($sql);
			$sth->execute(['metal_warehouse', $level]);
			return $sth->fetchColumn();
		}
		
		// узнать бонус к опыту от тренировочного зала
		public static function get_training_hall_bonus_exp_percent($level)
		{
			$sql = 'select bonus from _clan_buildings_levels where building = ? and level = ? limit 1';
			$sth = \db::$pdo->prepare($sql);
			$sth->execute(['training_hall', $level]);
			return $sth->fetchColumn();
		}
		
		// может ли постройка быть улучшена
		public static function is_building_increase($id_clan, $building)
		{
			$clan_bucks = clan_coffers::get_bucks($id_clan);
			$clan_metal = clan_coffers::get_metal($id_clan);
			$clan_mutagen = clan_coffers::get_mutagen($id_clan);
			
			$level = db\clan_building::building_level_get($id_clan, $building);
			$level_max = self::get_building_level_max($building);
			$price = self::get_building_price_improv($building, $level);
			
			if ($level >= $level_max) { // если постройка уже улучшена полностью
				return false;
			}
			if ($clan_bucks < $price['bucks'] || $clan_metal < $price['metal'] || $clan_mutagen < $price['mutagen']) { // если в казне недостаточно средств
				return false;
			} return true;
		}
		
		// повысить уровень постройки со всеми вытекающими
		public static function building_level_increase($id_clan, $building)
		{
			$level = db\clan_building::building_level_get($id_clan, $building);
			if ($building == 'residence') {
				$p = [
					'health' => self::BUILDING_RESIDENCE_USER_PARAM_BONUS_EACH_LEVEL,
					'power' => self::BUILDING_RESIDENCE_USER_PARAM_BONUS_EACH_LEVEL,
					'agil' => self::BUILDING_RESIDENCE_USER_PARAM_BONUS_EACH_LEVEL,
					'intell' => self::BUILDING_RESIDENCE_USER_PARAM_BONUS_EACH_LEVEL
				];
				$ids = clan::get_clan_part_id_by_id_clan($id_clan);
				if ($ids) {
					$sql = 'update users_combat set health = health + ?, power = power + ?, agil = agil + ?, intell = intell + ? where id_user = ? limit 1';
					$param = (object) $p;
					$clan_name = clan::get_clan_name_by_id_clan($id_clan);
					$clan_link = subord_url('clan', "id_clan=$id_clan");
					$jtitle = 'Клан: «%s»';
					$jtitle = sprintf($jtitle, "<a href='$clan_link'>$clan_name</a>");
					$jtext = 'Постройка «%s» была улучшена на уровень <b>%s</b>. Ваша выносливость, сила, ловкость и интеллект увеличились на <b class="limeGreen">%s единиц</b>';
					$jtext = sprintf($jtext, 'Резиденция', $level + 1, self::BUILDING_RESIDENCE_USER_PARAM_BONUS_EACH_LEVEL);
					foreach ($ids as $k) {
						$id_user = $k[0];
						$sth = \db::$pdo->prepare($sql);
						$sth->execute([$param->health, $param->power, $param->agil, $param->intell, $id_user]);
						// записать в личный журнал
						\journal::record_add(['title' => $jtitle, 'text' => $jtext], $id_user);
					}
				}
			}
	
			$price = self::get_building_price_improv($building, $level);
			clan_coffers::bucks_refill($id_clan, -$price['bucks']);
			clan_coffers::metal_refill($id_clan, -$price['metal']);
			clan_coffers::mutagen_refill($id_clan, -$price['mutagen']);
			db\clan_building::building_level_increase($id_clan, $building); // поднять уровень на 1
		}
		
		// создать записи со зданиями клана в бд
		public static function building_record_create($id_clan)
		{
			$sql = 'insert into clan_building set id_clan = ?, residence_level = ?, scientific_complex_level = ?, repository_level = ?, metal_warehouse_level = ?, training_hall_level = ?';
			$sth = db::$pdo->prepare($sql);
			$sth->execute([
				$id_clan,
				0, 0, 0, 0, 0
			]);
		}
	}
	
	// класс работы игрока с постройками клана
	class user_clan_building
	{
		// узнать бонус к мутагену от научного комплекса по уровню комплекса по id-игрока
		public static function get_scientific_complex_bonus_mutagen($id_user)
		{
			if ( ! clan::is_user_clan($id_user)) { // если игрок не в клане
				return 0;
			}
			$id_clan = clan::get_id_clan_by_id_user($id_user);
			$sc_level = db\clan_building::building_level_get($id_clan, 'scientific_complex');
			return self::get_scientific_complex_bonus_mutagen($sc_level);
		}
		
		// узнать бонус к баксам от хранилища по id-игрока
		public static function get_repository_bonus_bucks_percent($id_user)
		{
			if ( ! clan::is_user_clan($id_user)) { // если игрок не в клане
				return 0;
			}
			$id_clan = clan::get_id_clan_by_id_user($id_user);
			$rep_level = db\clan_building::building_level_get($id_clan, 'repository');
			return clan_building::get_repository_bonus_bucks_percent($rep_level);
			
		}
		
		// узнать бонус к металу от металлобазы по id-игрока
		public static function get_metal_warehouse_bonus_metal_percent($id_user)
		{
			if ( ! clan::is_user_clan($id_user)) { // если игрок не в клане
				return 0;
			}
			$id_clan = clan::get_id_clan_by_id_user($id_user);
			$mw_level = db\clan_building::building_level_get($id_clan, 'metal_warehouse');
			return clan_building::get_metal_warehouse_bonus_metal_percent($mw_level);
		}
		
		// узнать бонус к опыту от тренировочного зала по id-игрока
		public static function get_training_hall_bonus_exp_percent($id_user)
		{
			if ( ! clan::is_user_clan($id_user)) { // если игрок не в клане
				return 0;
			}
			$id_clan = clan::get_id_clan_by_id_user($id_user);
			$mw_level = db\clan_building::building_level_get($id_clan, 'training_hall');
			return clan_building::get_training_hall_bonus_exp_percent($mw_level);
		}
		
		// узнать достаточно ли привилегий у id_user для улучшения клановых построек
		public static function is_user_privilege_building_increase($user_id = false)
		{
			\user_id($user_id);
			$rank = clan::get_user_clan_rank($user_id);
			return package('clan-privilege-building-up')[$rank];
		}
		
		// может ли игрок улучшить постройку
		public static function is_user_building_increase($clan_id, $building, $user_id = false)
		{
			\user_id($user_id);
			if ( ! clan::is_user_clan($user_id, $clan_id)) { // если игрок не в этом клане
				return false;
			}
			if ( ! self::is_user_privilege_building_increase($user_id)) { // привилегий недостаточно
				return false;
			}
			if ( ! clan_building::is_building_increase($clan_id, $building)) { // если постройка не может быть улучшена по каким-то причинам
				return false;
			} return true;
		}
		
		// поднять уровень постройки
		public static function building_level_increase($clan_id, $building, $user_id = false)
		{
			\user_id($user_id);
			if ( ! self::is_user_building_increase($clan_id, $building, $user_id)) {
				return false;
			}
			clan_building::building_level_increase($clan_id, $building);
			$building_name = translate::clan_building_name($building);
			$user_name = user::login_get($user_id);
			$user_link = '<a href="%s">%s</a>';
			$user_link = sprintf($user_link, subord_url('profile', "id_user=$user_id"), $user_name);
			$text = "$user_link поднял(/а) уровень постройки «{$building_name}»";
			clan_journal::record_add($text, $clan_id);
			
			notf::set_notf('Уровень постройки поднят.', 'default');
			return true;
		}
	}
}