<?php

namespace db
{
	class users_journal extends \_tbl_line
	{
		protected static $table = 'users_journal';
		
		public static function table_get()
		{
			return self::$table;
		}
		
		public static function record_add($field, $repl) // array, array
		{
			$field_str = implode(', ', $field);
			self::add($field_str, $repl);
		}
		
		public static function count($user_id)
		{
			return self::get('count(*)', 'where id_user = ?', [$user_id]);
		}
		
		public static function count_new($user_id)
		{
			return self::get('count(*)', 'where id_user = ? and is_read = ?', [$user_id, 0]);
		}
		
		public static function clean($user_id)
		{
			$sql = 'delete from users_journal where id_user = ?';
			$sth = \db::$pdo->prepare($sql);
			$sth->execute([$user_id]);
		}
		
		public static function mark_as_read($user_id)
		{
			self::set('is_read = ?', ['where id_user = ?', ''], [1, $user_id]);
		}
	}
}

namespace
{
	class journal
	{
		// $data['title'], $data['text'], $data['timestamp'] ...
		public static function record_add($data, $user_id) // array
		{
			$field = [
				'id_user = ?',
				'title = ?',
				'text = ?',
				'timestamp = ?'
			];
			if ( ! isset($data['title'])) {
				$data['title'] = null;
			}
			if ( ! isset($data['timestamp'])) {
				$data['timestamp'] = time();
			}
			$repl = [
				$user_id,
				$data['title'],
				$data['text'],
				$data['timestamp']
			];
			db\users_journal::record_add($field, $repl);
		}
		
		// отметить как прочитанные
		public static function mark_as_read($user_id)
		{
			db\users_journal::mark_as_read($user_id);
		}
		
		public static function clean($user_id)
		{
			db\users_journal::clean($user_id);
		}
	}
	
	class journal_model
	{
		public function journal($user_id)
		{
			$res = self::journal_get($user_id);
			return [
				'pag' => $res['pag'],
				'journal' => $res['journal']
			];
		}
		
		public function journal_get($user_id)
		{
			$table = db\users_journal::table_get();
			$pag = new pagination;
			$num = db\users_journal::count($user_id);
			$limit = $pag->start($num, 10);
			$sql = "select * from $table where id_user = ? order by id desc limit $limit[start], $limit[limit]";
			$sth = db::$pdo->prepare($sql);
			$sth->execute([$user_id]);
			$journal = $sth->fetchAll(PDO::FETCH_ASSOC);
			return [
				'pag' => $pag,
				'journal' => $journal
			];
		}
	}
}