<?php

namespace ampoule;

class ampoule
{
	private $table_repository_ampoule = '_ampoule';
	
	/* получить максимальный уровень ампулы */
	public function get_level_max()
	{
		$sql = "select max(level) from $this->table_repository_ampoule limit 1";
		$sth = \db::$pdo->query($sql);
		return $sth->fetchColumn();
	}
	
	/* получить цену улучшения ампулы */
	public function get_price_mutagen($ampoule_level)
	{
		$sql = "select price_mutagen from $this->table_repository_ampoule where level = ? limit 1";
		$sth = \db::$pdo->prepare($sql);
		$sth->execute([$ampoule_level]);
		$price_mutagen = $sth->fetchColumn();
		
		$stock = new \stock\stock();
		$discount_ampoule = $stock->get_discount_ampoule();
		if ($discount_ampoule > 0) {
			$price_mutagen_minus = (int) floor(percentage($price_mutagen, $discount_ampoule));
			$price_mutagen -= $price_mutagen_minus;
		}
		return $price_mutagen;
	}
	
	/* получить параметры ампулы */
	public function get_param($type, $level)
	{
		$sql = "select health, power, agil, intell from $this->table_repository_ampoule where type = ? and level = ? limit 1";
		$sth = \db::$pdo->prepare($sql);
		$sth->execute([$type, $level]);
		$param = $sth->fetchObject();
		return [
			'health' => $param->health,
			'power' => $param->power,
			'agil' => $param->agil,
			'intell' => $param->intell
		];
	}
}

class user_ampoule
{
	public $user_id;
	
	private $table_users_ampoule = 'users_lab';
	
	public function __construct($user_id)
	{
		$this->user_id = $user_id;
	}
	
	/* улучшить ампулу */
	public function improve($type)
	{
		$improver = new improver($this->user_id, $type);
		return $improver->improve();
	}
	
	/* получить уровень ампулы */
	public function get_level($type)
	{
		$sql = "select ampoule_$type from $this->table_users_ampoule where id_user = ? limit 1";
		$sth = \db::$pdo->prepare($sql);
		$sth->execute([$this->user_id]);
		return $sth->fetchColumn();
	}
	
	/* увеличить уровень ампулы */
	public function increase_level($type)
	{
		$sql = "update $this->table_users_ampoule set ampoule_$type = ampoule_$type + 1 where id_user = ? limit 1";
		$sth = \db::$pdo->prepare($sql);
		return $sth->execute([$this->user_id]);
	}
}

class improver
{
	/* (int) кто улучшает: id-игрока */
	public $user_id;
	
	/* (string) тип ампулы которая улучшается: красная (+вын.), жёлтая (+сил.), синяя (+ловк.), зелёная (инт.) */
	public $type;
	
	public function __construct($user_id, $type)
	{
		$this->user_id = $user_id;
		$this->type = $type;
	}
	
	public function improve()
	{
		$user_id = $this->user_id;
		$type = $this->type;
		$user_ampoule = new user_ampoule($user_id);
		$ampoule = new ampoule();
		
		$user_ampoule_level = $user_ampoule->get_level($type);
		$ampoule_level_max = $ampoule->get_level_max();
		if ($user_ampoule_level >= $ampoule_level_max) {
			return false;
		}
		
		$user_mutagen = \currency::mutagen_get($this->user_id);
		$ampoule_price_mutagen = $ampoule->get_price_mutagen($user_ampoule_level);
		if ($user_mutagen < $ampoule_price_mutagen) {
			return false;
		}
		
		/* улучшить ампулу */
		$user_ampoule->increase_level($type);
		
		/* снять мутаген */
		$currency = new \currency($user_id);
		$currency->bonuses = false;
		$currency->base = -$ampoule_price_mutagen;
		$currency->mutagen_add();
		
		/* увеличить параметры игрока */
		$param = $ampoule->get_param($type, $user_ampoule_level + 1);
		\uc::update_param_all($user_id, $param);
		return true;
	}
}