<?php

class lab
{	
	// получить столбец mutagen из users_lab по id игрока
	public static function get_mutagen_by_id($id_user)
	{
		$sql = 'select mutagen from users_lab where id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_user]);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// получить время
	public static function get_mutagen_time_by_id($id_user)
	{
		$sql = 'select mutagen_time from users_lab where id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_user]);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// изменить количество лабораторного мутагена
	public static function mutagen_update($id_user, $mutagen_num)
	{
		$sql = 'update users_lab set mutagen = mutagen + ? where id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$mutagen_num, $id_user]);
	}
	
	// установить новое время
	public static function mutagen_time_update($id_user, $time = false)
	{
		if ($time === false) {
			$time = time();
		}
		$sql = 'update users_lab set mutagen_time = ? where id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$time, $id_user]);
	}
	
	// восстановление мутагена в лаборатории
	public static function mutagen_recovery($id_user)
	{
		$mutagen_num = self::get_mutagen_by_id($id_user);
		if ($mutagen_num < 1) {
			$time = self::get_mutagen_time_by_id($id_user);
			if (day_new($time, 1)) { // если наступил новый день
				self::mutagen_update($id_user, self::mutagen_generate($id_user)); // восстановить мутаген
			}
		}
	}
	
	// генерировать мутаген
	public static function mutagen_generate($id_user)
	{
		$ll = self::get_lab_level_by_id($id_user);
		$m = self::get_mutagen_by_lab_level($ll);
		if (clan::is_user_clan($id_user)) { // если игрок в клане
			$id_clan = clan::get_id_clan_by_id_user($id_user);
			$sc_level = db\clan_building::building_level_get($id_clan, 'scientific_complex');
			$m += clan_building::get_scientific_complex_bonus_mutagen($sc_level);
		}
		return $m;
	}
	
	// узнать уровень лаборатории
	public static function get_lab_level_by_id($id_user)
	{
		$sql = "select lab_level from users_lab where id_user = ? limit 1";
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_user]);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// узнать максимальный уровень лаборатории
	public static function get_lab_level_max()
	{
		$sql = 'select max(level) from _levels_lab limit 1';
		$sth = \db::$pdo->query($sql);
		return $sth->fetchColumn();
	}
	
	// узнать количество вырабатываемого мутагена по уровню лаборатории
	public static function get_mutagen_by_lab_level($level)
	{
		$sql = 'select bonus_mutagen from _levels_lab where level = ? limit 1';
		$sth = \db::$pdo->prepare($sql);
		$sth->execute([$level]);
		return $sth->fetchColumn();
	}
	
	// узнать цену улучшения по уровню лаборатории
	public static function get_price_by_lab_level($level, $price_type)
	{
		$sql = 'select price_bucks as bucks, price_metal as metal from _levels_lab where level = ? limit 1';
		$sth = \db::$pdo->prepare($sql);
		$sth->execute([$level]);
		$prices = $sth->fetch(\PDO::FETCH_ASSOC);
		return $prices[$price_type];
	}
	
	// изменить уровень лаболатории
	public static function lab_level_update($id_user, $num)
	{
		$sql = "update users_lab set lab_level = lab_level + ? where id_user = ? limit 1";
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$num, $id_user]);
	}
}

class lab_model
{
	public function lab()
	{
		$user_id = ID;
		$user_ampoule = new \ampoule\user_ampoule($user_id);
		$ampoule = new \ampoule\ampoule();
		
		$ampoule_level = [
			'red' => $user_ampoule->get_level('red'),
			'yellow' => $user_ampoule->get_level('yellow'),
			'blue' => $user_ampoule->get_level('blue'),
			'green' => $user_ampoule->get_level('green')
		];
		$ampoule_price = [
			'red' => $ampoule->get_price_mutagen($ampoule_level['red']),
			'yellow' => $ampoule->get_price_mutagen($ampoule_level['yellow']),
			'blue' => $ampoule->get_price_mutagen($ampoule_level['blue']),
			'green' => $ampoule->get_price_mutagen($ampoule_level['green'])
		];
		
		if ($ampoule_price['red'] < 1) $ampoule_price['red'] = null;
		if ($ampoule_price['yellow'] < 1) $ampoule_price['yellow'] = null;
		if ($ampoule_price['blue'] < 1) $ampoule_price['blue'] = null;
		if ($ampoule_price['green'] < 1) $ampoule_price['green'] = null;
		
		$lab_level = lab::get_lab_level_by_id($user_id);
		$lab_level_max = lab::get_lab_level_max();
		$lab_level_price = [
			'bucks' => lab::get_price_by_lab_level($lab_level, 'bucks'),
			'metal' => lab::get_price_by_lab_level($lab_level, 'metal')
		];
		$param_name = translate::param_name();
		$mutagen = (int) floor(lab::get_mutagen_by_id($user_id));
		return [
			'mutagen' => $mutagen,
			'ampoule_level' => $ampoule_level,
			'ampoule_price' => $ampoule_price,
			'param_name' => $param_name
		];
	}
	
	public function llab()
	{
		$lab_level = lab::get_lab_level_by_id(ID);
		$lab_level_max = lab::get_lab_level_max();
		$lab_level_mutagen = lab::mutagen_generate(ID);
		$lab_level_price = [
			'bucks' => lab::get_price_by_lab_level($lab_level, 'bucks'),
			'metal' => lab::get_price_by_lab_level($lab_level, 'metal')
		];
		return [
			'lab_level' => $lab_level,
			'lab_level_max' => $lab_level_max,
			'lab_level_mutagen' => $lab_level_mutagen,
			'lab_level_price' => $lab_level_price
		];
	}
}