<?php

namespace db
{
	class users_mail extends \_tbl_line
	{
		protected static $table = 'users_mail';
		
		public static function table_get()
		{
			return self::$table;
		}
		
		public static function contacts_count($user_id)
		{
			return self::get('count(*)', 'where id_user = ? group by id_interlocutor', [$user_id]);
		}
		
		// если interlocutor_id указан, то считает кол-во сообщенией с этим собеседником, иначе все
		public static function messages_count($user_id, $interlocutor_id = false)
		{
			if ($interlocutor_id === false) {
				return self::get('count(*)', 'where id_user = ?', [$user_id]);
			} return self::get('count(*)', 'where id_user = ? and id_interlocutor = ?', [$user_id, $interlocutor_id]);
		}
		
		// посчитать непрочитанные сообщения все или с конкретным собеседником
		public static function messages_not_read_count($user_id, $interlocutor_id = false)
		{
			if ($interlocutor_id === false) {
				return self::get('count(*)', 'where id_user = ? and is_read = 0', [$user_id]);
			} return self::get('count(*)', 'where id_user = ? and id_interlocutor = ? and is_read = 0', [$user_id, $interlocutor_id]);
		}
		
		public static function interlocutor_isset($user_id, $interlocutor_id)
		{
			if (self::get('count(*)', 'where id_user = ? and id_interlocutor = ?', [$user_id, $interlocutor_id]) > 0) {
				return true;
			} return false;
		}
		
		// получить текст последнего сообщения между id_user_1 и id_user_2
		public static function message_last_get($user_1_id, $user_2_id)
		{
			return self::get('text', 'where id_user = ? and id_interlocutor = ? order by id desc', [$user_1_id, $user_2_id]);
		}
		
		public static function messages_mark_as_read($user_id, $interlocutor_id)
		{
			return self::set('is_read = 1', ['where id_user = ? and id_interlocutor = ?', ''], [$user_id, $interlocutor_id]);
		}
		
		// очистить почту игрока
		public static function mail_clean($user_id)
		{
			$sql = 'delete from ' . self::table_get() . ' where id_user = ?';
			$sth = \db::$pdo->prepare($sql);
			return $sth->execute([$user_id]);
		}
		
		// удалить переписку с собеседником
		public static function correspondence_delete($user_id, $interlocutor_id)
		{
			$sql = 'delete from ' . self::table_get() . ' where id_user = ? and id_interlocutor = ?';
			$sth = \db::$pdo->prepare($sql);
			return $sth->execute([$user_id, $interlocutor_id]);
		}
	}
}

namespace
{
	class mail
	{
		public $table;
		public $user_id, $interlocutor_id, $sender_id, $recipient_id, $text, $timestamp;
		
		public function __construct()
		{
			$this->table = db\users_mail::table_get();
		}
		
		public function message_add()
		{
			$sql = "insert into " . $this->table . " set id_user = ?, id_interlocutor = ?, id_sender = ?, id_recipient = ?, text = ?, timestamp = ?, is_read = ?";
			$sth = db::$pdo->prepare($sql);
			$sth->execute([$this->user_id, $this->interlocutor_id, $this->sender_id, $this->recipient_id, $this->text, $this->timestamp, 1]);
			$sth->execute([$this->interlocutor_id, $this->user_id, $this->sender_id, $this->recipient_id, $this->text, $this->timestamp, 0]);
		}
		
		public function clean()
		{
			\db\users_mail::mail_clean($this->user_id);
		}
		
		public function correspondence_delete()
		{
			\db\users_mail::correspondence_delete($this->user_id, $this->interlocutor_id);
		}
	}
	
	class mail_model
	{
		public function mail($data)
		{
			$user_id = $data['user_id'];
			$res = $this->contacts_get($user_id);
			return [
				'pag' => $res['pag'],
				'contacts' => $res['contacts'],
				'clean_confirm' => $data['clean_confirm']
			];
		}
		
		public function correspondence($data)
		{
			$user_id = $data['user_id'];
			$interlocutor_id = $data['interlocutor_id'];
			$res = $this->correspondence_get($user_id, $interlocutor_id);
			$interlocutor_name = user::login_get($interlocutor_id);
			return [
				'pag' => $res['pag'],
				'correspondence' => $res['correspondence'],
				'interlocutor_id' => $interlocutor_id,
				'interlocutor_name' => $interlocutor_name,
				'correspondence_delete_confirm' => $data['correspondence_delete_confirm']
			];
		}
		
		// достать список контактов
		public function contacts_get($user_id)
		{
			$table = db\users_mail::table_get($user_id);
			$pag = new pagination;
			$limit = $pag->start(db\users_mail::contacts_count($user_id), 10);
			$sql = "select max(id) as max_id, id_interlocutor, min(is_read) as min_is_read from $table where id_user = ? group by id_interlocutor order by min_is_read asc, max_id desc limit $limit[start], $limit[limit]";
			$sth = db::$pdo->prepare($sql);
			$sth->execute([$user_id]);
			$contacts = $sth->fetchAll(PDO::FETCH_ASSOC);
			foreach ($contacts as $key => $val) {
				$contacts[$key]['interlocutor_name'] = user::login_get($val['id_interlocutor']);
				$contacts[$key]['interlocutor_gender'] = user::gender_get($val['id_interlocutor']);
				$contacts[$key]['message_last'] = db\users_mail::message_last_get($user_id, $val['id_interlocutor']);
				$contacts[$key]['messages_new_num'] = db\users_mail::messages_not_read_count($user_id, $val['id_interlocutor']);
			}
			return [
				'pag' => $pag,
				'contacts' => $contacts
			];
		}
		
		public function correspondence_get($user_id, $interlocutor_id)
		{
			$table = db\users_mail::table_get($user_id);
			$pag = new pagination;
			$limit = $pag->start(db\users_mail::messages_count($user_id, $interlocutor_id), 10);
			$sql = "select id_interlocutor, id_sender, id_recipient, text, timestamp, is_read from $table where id_user = ? and id_interlocutor = ? order by id desc limit $limit[start], $limit[limit]";
			$sth = db::$pdo->prepare($sql);
			$sth->execute([$user_id, $interlocutor_id]);
			$correspondence = $sth->fetchAll(PDO::FETCH_ASSOC);
			foreach ($correspondence as $key => $val) {
				$correspondence[$key]['interlocutor_name'] = ($val['id_sender'] == $user_id) ? false : user::login_get($val['id_sender']);
				$correspondence[$key]['interlocutor_gender'] = ($val['id_sender'] == $user_id) ? user::gender_get($user_id) : user::gender_get($val['id_sender']);;
				$correspondence[$key]['seconds_left'] = time() - $val['timestamp'];
			}
			return [
				'pag' => $pag,
				'correspondence' => $correspondence
			];
		}
	}
}