<?php

namespace ml2;

class command
{
	const PART_MAX = 4; // макс. участников в команде
	const STATUS_EXPECT = 1; // статус - ожидание (сбор команды)
	const STATUS_BATTLE = 2; // статус - бой
	const STATUS_VICTORY = 3; // победа
	const STATUS_LOST = 4; // поражение
	
	const ACCESS_OPEN = 1; // открытый доступ
	const ACCESS_CLOSE = 2; // закрытый доступ
	
	public
		$user_1_id = null,
		$user_2_id = null,
		$user_3_id = null,
		$status = 1;
	public 
		$location_id,
		$leader_id;
	protected $table = 'ml2_commands';
	protected $last_insert_id;
	
	public function create()
	{
		$sql = 'insert into %s set id_location = ?, id_leader = ?, id_user_1 = ?, id_user_2 = ?, id_user_3 = ?, status = ?';
		$sql = sprintf($sql, $this->table);
		$sth = \db::$pdo->prepare($sql);
		$sth->execute([
			$this->location_id,
			$this->leader_id,
			$this->user_1_id,
			$this->user_2_id,
			$this->user_3_id,
			$this->status
		]);
		$this->last_insert_id = \db::$pdo->lastInsertId();
	}
	
	public function delete($id)
	{
		\db\ml2_commands::delete($id);
	}
	
	public function last_insert_id_get()
	{
		return $this->last_insert_id;
	}
	
	public function battle_throw($command_id)
	{
		\db\ml2_commands::status_set(self::STATUS_BATTLE, $command_id);
		// создать игроков
		$battle = new battle;
		$battle->battle_id = \db\ml2_commands::battle_id_get($command_id);
		$battle->users_id = \db\ml2_commands::parts_id_get($command_id);
		$battle->create();
	}
}

class user_command extends command
{
	public function create()
	{
		if (\db\ml2_commands::in_command($this->leader_id)) { // уже в команде
			return;
		}
		parent::create();
	}
	
	public function delete($id)
	{
		if ( ! \db\ml2_commands::is_command($id)) {
			return;
		}
		parent::delete($id);
	}
	
	public function start($user_id)
	{
		if ( ! \db\ml2_commands::is_leader($user_id)) {
			return;
		}
		$command_id = \db\ml2_commands::command_id_get($user_id);
		if (\db\ml2_commands::status_get($command_id) != self::STATUS_EXPECT) {
			return;
		}
		$this->battle_throw($command_id);
		
		// удалить все приглашения в эту команду !
		\db\ml2_invite::invite_to_command_delete($command_id);
		
		$parts = \db\ml2_commands::parts_id_get($command_id);
		foreach ($parts as $user_id) {
			/* почистить лог для всех участников */
			$lbc = new \log_battle_clean;
			$lbc->user_id = $user_id;
			$lbc->start();
			/* создать запись со статистикой для всех участников */
			\db::$pdo->query("insert into ml2_statistic set id_user = $user_id, id_battle = $command_id");
		}
		
		
	}
}

class command_find // найти команду
{
	public $user_id, $location_id;
	
	public function tap()
	{
		if ( ! \db\ml2_locations::is_open($this->user_id, $this->location_id)) { // этот канал пройден
			return;
		}
		if (\db\ml2_commands::in_command($this->user_id)) { // вы уже в команде
			return;
		}
		// не искать команды лидер которых не обновлял страницу более минуты
		$time = time() - 60; // задержка минута
		$sql = '
			select
				mc.id
			from
				ml2_commands mc
			left join
				users_online uo
			on
				uo.id_user = mc.id_leader
				and uo.time >= ?
			where
				mc.id_leader = uo.id_user
				and (mc.id_location = ? and mc.access = ?)
				and (mc.id_user_1 is null or mc.id_user_2 is null or mc.id_user_3 is null)
				and mc.status = ?
			';
		$sth = \db::$pdo->prepare($sql);
		$sth->execute([$time, $this->location_id, command::ACCESS_OPEN, command::STATUS_EXPECT]);
		$commands = $sth->fetchAll(\PDO::FETCH_NUM);
		if ( ! $commands) {
			\notf::set_notf('Команда не найдена.', 'error');
			return;
		}
		$command_id = $commands[array_rand($commands)][0];
		$cua = new command_user_add;
		$cua->user_id = $this->user_id;
		$cua->command_id = $command_id;
		$cua->execute();
		return true;
	}
}

/* покинуть команду */
class command_leave
{
	public $user_id;
	
	public function tap()
	{
		if ( ! \db\ml2_commands::in_command($this->user_id)) {
			return;
		}
		$command_id = \db\ml2_commands::command_id_get($this->user_id);
		if (\db\ml2_commands::is_leader($this->user_id)) {
			$sql = 'select id_user_1, id_user_2, id_user_3 from ml2_commands where id = ? limit 1';
			$sth = \db::$pdo->prepare($sql);
			$sth->execute([$command_id]);
			$part = $sth->fetch(\PDO::FETCH_ASSOC);
			$whom = null;
			if ( ! is_null($part['id_user_1'])) $whom = $part['id_user_1'];
			elseif ( ! is_null($part['id_user_2'])) $whom = $part['id_user_2'];
			elseif ( ! is_null($part['id_user_3'])) $whom = $part['id_user_3'];
			if (is_null($whom)) { // если некому передать лидерство
				$command_destroy = new command_destroy;
				$command_destroy->command_id = $command_id;
				$command_destroy->execute();
			}
			else {
				\db\ml2_commands::part_expel($whom); // удаляем из команды того кто будет новым лидером
				\db\ml2_commands::leader_exchange($command_id, $whom); // ставим его лидером
			}
		}
		
		\db\ml2_commands::part_expel($this->user_id); // удалить из команды
		\db\ml2_part::user_delete($this->user_id); // удалить из боя
		\db\ml2_statistic::delete($this->user_id, $command_id); // удалить статистику
	}
}

/* распустить команду */
class command_dissolve
{
	public $user_id;
	
	public function tap()
	{
		if ( ! \db\ml2_commands::in_command($this->user_id)) {
			return;
		}
		if ( ! \db\ml2_commands::is_leader($this->user_id)) {
			return;
		}
		$command_id = \db\ml2_commands::command_id_get($this->user_id);
		if ( ! \db\ml2_commands::status_get($command_id) == command::STATUS_EXPECT) {
			return;
		}
		$command_destroy = new command_destroy;
		$command_destroy->command_id = $command_id;
		$command_destroy->execute();
	}
}

/* исключить из команды */
class command_deleted
{
	public $who_user_id; // кто исключает
	public $whom_user_id; // кого исключает
	
	public function execute()
	{
		if ( ! \db\ml2_commands::is_leader($this->who_user_id)) {
			return;
		}
		if ($this->who_user_id == $this->whom_user_id) {
			return;
		}
		if ( ! \db\ml2_commands::in_one_command([$this->who_user_id, $this->whom_user_id])) {
			return;
		}
		$command_id = \db\ml2_commands::command_id_get($this->who_user_id);
		if (\db\ml2_commands::status_get($command_id) != command::STATUS_EXPECT) {
			return;
		}
		\db\ml2_commands::part_expel($this->whom_user_id);
	}
}

/* открыть/закрыть доступ */
class command_access
{
	public $user_id; // кто меняет доступ
	
	public function execute()
	{
		if ( ! \db\ml2_commands::is_leader($this->user_id)) {
			return;
		}
		$command_id = \db\ml2_commands::command_id_get($this->user_id);
		if (\db\ml2_commands::status_get($command_id) != command::STATUS_EXPECT)  {
			return;
		}
		$access = \db\ml2_commands::access_get($command_id);
		if ($access == command::ACCESS_OPEN) {
			return \db\ml2_commands::access_set($command_id, command::ACCESS_CLOSE);
		}
		if ($access == command::ACCESS_CLOSE) {
			return \db\ml2_commands::access_set($command_id, command::ACCESS_OPEN);
		}
	}
}

/* добавление нового участника в команду */
class command_user_add
{
	public $user_id;
	public $command_id;
	
	public function execute()
	{
		if (\db\ml2_commands::in_command($this->user_id)) {
			return;
		}
		$place = $this->place_pick_up();
		\db\ml2_commands::at_command($this->command_id, $this->user_id, $place);
	}
	
	public function place_pick_up()
	{
		$sql = "select id_user_1, id_user_2, id_user_3 from ml2_commands where id = $this->command_id limit 1";
		$sth = \db::$pdo->query($sql);
		$places = $sth->fetch(\PDO::FETCH_ASSOC);
		if (is_null($places['id_user_1'])) {
			return 1;
		}
		elseif (is_null($places['id_user_2'])) {
			return 2;
		}
		elseif (is_null($places['id_user_3'])) {
			return 3;
		}
		else {
			return false;
		}
	}
}

/* уничтожение команды */
class command_destroy
{
	public $command_id;
	
	public function execute()
	{
		if ( ! \db\ml2_commands::is_command($this->command_id)) {
			return;
		}
		$battle_id = \db\ml2_commands::battle_id_get($this->command_id);
		\db\ml2_commands::delete($this->command_id); // удаляем команду
		\db\ml2_invite::invite_to_command_delete($this->command_id); // удалить приглашения
		\db\ml2_part::battle_delete($battle_id); // чистим бой
		\db\ml2_statistic::battle_delete($battle_id); // чистим статистику
		
		$command_chat = new \ml2\command_chat($this->command_id);
		$command_chat->delete_chat(); // уничтожить чат
	}
}