<?php

class news
{
	public $user_id;
	public $title;
	public $text;
	
	public function create()
	{
		if ( ! $this->data_handler()) {
			return false;
		}
		$title = $this->title;
		$text = $this->text;
		
		$sql = 'insert into news set title = ?, text = ?, created_timestamp = ?, id_creator = ?';
		$sth = db::$pdo->prepare($sql);
		notf::set_notf('Новость добавлена!');
		$sth->execute([$title, $text, time(), $this->user_id]);
		return db::$pdo->lastInsertID();
	}
	
	public function edit($id)
	{
		if ( ! $this->data_handler()) {
			return false;
		}
		if ( ! self::is_privilege_news_edit($this->user_id)) {
			return false;
		}
		$title = $this->title;
		$text = $this->text;
		
		$sql = 'update news set title = ?, text = ? where id = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		notf::set_notf('Новость изменена!');
		return $sth->execute([$title, $text, $id]);
	}
	
	public function delete($id)
	{
		if ( ! self::isset_news($id)) {
			return false;
		}
		if ( ! self::is_privilege_news_delete($this->user_id)) {
			return false;
		}
		$sql = 'delete from news where id = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		notf::set_notf('Новость удалена!');
		return $sth->execute([$id]);
	}
	
	private function data_handler()
	{
		if (empty($this->title) || empty($this->text)) {
			notf::set_notf('Оба поля обязательны для заполнения!', 'error');
			return false;
		} return true;
	}
	
	/* получить новость с id */
	public static function get_news($news_id)
	{
		$sql = 'select * from news where id = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$news_id]);
		return $sth->fetch(PDO::FETCH_ASSOC);
	}
	
	/* получить n последних новостей */
	public static function get_last_news($n)
	{
		$pag = new pagin;
		$limit = $pag->start(self::count_news(), $n);
		$sql = "select * from news order by id desc limit $limit[start], $limit[limit]";
		$sth = db::$pdo->prepare($sql);
		$sth->execute();
		return [
			'pag' => $pag,
			'news' => $sth->fetchAll(PDO::FETCH_ASSOC)
		];
	}
	
	public static function count_news()
	{
		$sth = db::$pdo->query('select count(*) from news limit 1');
		return $sth->fetchColumn();
	}
	
	public static function isset_news($id)
	{
		$sql = 'select count(*) from news where id = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id]);
		return $sth->fetchColumn();
	}
	
	/* может ли пользователь создавать новость */
	public static function is_privilege_news_create($user_id)
	{
		// получить position пользователя
		$position = user::position_get($user_id);
		if ($position > 1) {
			return true;
		} return false;
	}
	
	/* может ли пользователь изменять новость */
	public static function is_privilege_news_edit($user_id)
	{
		// получить position пользователя
		$position = user::position_get($user_id);
		if ($position > 1) {
			return true;
		} return false;
	}
	
	/* может ли пользователь удалять новость */
	public static function is_privilege_news_delete($user_id)
	{
		// получить position пользователя
		$position = user::position_get($user_id);
		if ($position > 1) {
			return true;
		} return false;
	}
	
	/* фильтр идентификатора новости (для безопасного запуска в бд) */
	public static function filter_news_id($id)
	{
		if ( ! is_numeric($id) || $id < 1) {
			return false;
		}
		if ( ! self::isset_news($id)) {
			return false;
		} return true;
	}
	
	/* получить id последней новости */
	public static function get_news_last_id()
	{
		$sql = 'select id from news order by id desc limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute();
		return $sth->fetchColumn();
	}
	
	/* получить id последней новости которая "была прочитана игроков" */
	public static function get_news_user_last_id($user_id)
	{
		$sql = 'select id_news_last from users where id = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$user_id]);
		return $sth->fetchColumn();
	}
	
	public static function update_news_user_last_id($user_id)
	{
		$last_id = self::get_news_last_id();
		if ( ! $last_id) {
			return false;
		}
		$user_last_id = self::get_news_user_last_id($user_id);
		if ($last_id == $user_last_id) {
			return false;
		}
		$sql = 'update users set id_news_last = ? where id = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		return $sth->execute([$last_id, $user_id]);
	}
}