<?php

class news_comments
{
	/* максимальная длина комментария */
	const COMMENT_MAX_LENGTH = 1000;
	
	public $news_id;
	public $comment_id;
	
	public $user_id;
	public $recipient_id;
	public $comment_text;
	
	public function __construct($news_id)
	{
		$this->news_id = $news_id;
	}
	
	/* добавление комментария */
	public function add_comment()
	{
		if ( ! is_string($this->comment_text)) {
			return false;
		}
		$len = mb_strlen($this->comment_text);
		if ($len < 1) {
			return false;
		}
		if ($len > self::COMMENT_MAX_LENGTH) {
			notf::set_notf('Длина комментария не должна превышать ' . self::COMMENT_MAX_LENGTH . " символов. У вас $len символов.", 'error');
			return false;
		}
		$comment_text = htmlspecialchars($this->comment_text);
		if ($this->recipient_id) {
			$recipient_id = $this->recipient_id;
			$title = '<a href="' . subord_url('profile', "id_user=$this->user_id") . '">' . user::login_get($this->user_id) . '</a> ответил/а на ваш комментарий.';
			$text = "<div>$comment_text</div>";
			$text .= '<a href="' . subord_url('news_detail', "id=$this->news_id") . '">Перейти</a>';
			journal::record_add(['title' => $title, 'text' => $text], $recipient_id);
		} else {
			$recipient_id = null;
		}
		$sql = 'insert into news_comments set id_news = ?, id_user = ?, id_recipient = ?, text = ?, timestamp = ?';
		$sth = db::$pdo->prepare($sql);
		$result = $sth->execute([$this->news_id, $this->user_id, $recipient_id, $comment_text, time()]);
		if ($result) {
			notf::set_notf('Комментарий добавлен.', 'default');
			return true;
		} return false;
	}
	
	/* удаление комментария */
	public function delete_comment()
	{
		if ( ! is_numeric($this->comment_id) && $this->comment_id < 1) {
			return false;
		}
		$is_privilege_comment_delete = self::is_privilege_comment_delete($this->user_id);
		if ( ! $is_privilege_comment_delete) {
			return false;
		}
		if ( ! self::isset_comment($this->comment_id, $this->news_id)) {
			notf::set_notf("Комментарий [$this->comment_id], не существует!", 'error');
			return false;
		}
		
		$sql = 'delete from news_comments where id = ? and id_news = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$result = $sth->execute([$this->comment_id, $this->news_id]);
		if ($result) {
			notf::set_notf("Комментарий [$this->comment_id] удалён!", 'error');
			return true;
		} return false;
	}
	
	/* получить n последних комментариев новости */
	public function get_last_comments($n)
	{
		$pag = new pagin;
		$limit = $pag->start(self::count_comments($this->news_id), $n);
		$sql = "select * from news_comments where id_news = ? order by id desc limit $limit[start], $limit[limit]";
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$this->news_id]);
		return [
			'pag' => $pag,
			'comments' => $sth->fetchAll(PDO::FETCH_ASSOC)
		];
	}
	
	/* посчитать комментарии */
	public static function count_comments($news_id)
	{
		$sql = 'select count(*) from news_comments where id_news = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$news_id]);
		return $sth->fetchColumn();
	}
	
	/* существует ли комментарий */
	public static function isset_comment($comment_id, $news_id)
	{
		$sql = 'select count(*) from news_comments where id = ? and id_news = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$comment_id, $news_id]);
		if ($sth->fetchColumn() > 0) {
			return true;
		} return false;
	}
	
	/* есть ли у пользователя права на удаление комментариев */
	public static function is_privilege_comment_delete($user_id)
	{
		$position = user::position_get($user_id);
		if ($position > 1) {
			return true;
		} return false;
	}
	
	/* получить id новости комментария */
	// public static function get_news_id_by_comment($comment_id)
	// {
		// $sql = 'select id_news from news_comments where id = ? limit 1';
		// $sth = db::$pdo->prepare($sql);
		// $sth->execute([$comment_id]);
		// return $sth->fetchColumn();
	// }
}