<?php

namespace db
{
	class ratings
	{
		public static function exp_count()
		{
			$sql = 'select count(*) from users_statistic where expr > 0 limit 1';
			$sth = \db::$pdo->query($sql);
			return $sth->fetch(\PDO::FETCH_NUM)[0];
		}
		
		public static function stats_count()
		{
			$sql = 'select count(*) from users_combat limit 1';
			$sth = \db::$pdo->query($sql);
			return $sth->fetch(\PDO::FETCH_NUM)[0];
		}
		
		public static function clan_count()
		{
			$sql = 'select count(*) from clan limit 1';
			$sth = \db::$pdo->query($sql);
			return $sth->fetch(\PDO::FETCH_NUM)[0];
		}
		
		//       
		public static function get_user_place_expr($user_id)
		{
			\db::$pdo->exec('set @n:=0;');
			$sql = '
				select * from (select @n := @n + 1, us.id_user as iduser, us.expr from users_statistic us where us.expr > 0 order by us.expr desc) x where iduser = ? limit 1
			';
			$sth = \db::$pdo->prepare($sql);
			$sth->execute([$user_id]);
			$row = $sth->fetchAll(\PDO::FETCH_NUM);
			if ($row) {
				return $row[0][0];
			} return false;
		}
		
		//       
		public static function get_user_place_param($user_id)
		{
			\db::$pdo->exec('set @n:=0;');
			$sql = '
				select * from (select @n := @n + 1, uc.id_user as iduser, uc.health + uc.power + uc.agil + uc.intell as param_sum, (select u.login from users u where u.id = uc.id_user) as login from users_combat uc order by param_sum desc, login asc) x where iduser = ? limit 1
			';
			$sth = \db::$pdo->prepare($sql);
			$sth->execute([$user_id]);
			$row = $sth->fetchAll(\PDO::FETCH_NUM);
			return $row[0][0];
		}
	}
}

namespace
{
	class ratings_model
	{
		public function ratings($type)
		{
			$method = "rating_{$type}_get";
			$res = $this->$method();
			return [
				'type' => $type,
				'rait' => $res['rating'],
				'pag' => $res['pag']
			];
		}
		
		public function ratings_clan($type)
		{
			$method = "rating_{$type}_get";
			$res = $this->$method();
			return [
				'type' => $type,
				'rait' => $res['rating'],
				'pag' => $res['pag']
			];
		}
		
		protected function rating_exp_get()
		{
			$on_page = 10; //   
			$on_which_user = ceil(db\ratings::get_user_place_expr(ID) / $on_page);
			$on_which_user = ($on_which_user < 1) ? 1 : $on_which_user;
			
			$pag = new pagination;
			$pag->page_default = $on_which_user;
			
			$limit = $pag->start(db\ratings::exp_count(), $on_page);
			
			
			$sql = "select us.expr as score, u.id as id_user, u.login from users_statistic us
			left join users u on u.id = us.id_user
			where us.expr > 0 order by us.expr desc limit $limit[start], $limit[limit]";
			$sth = db::$pdo->query($sql);
			$rating = $sth->fetchAll(PDO::FETCH_ASSOC);
			return [
				'pag' => $pag,
				'rating' => $rating
			];
		}
		
		protected function rating_stats_get()
		{
			$on_page = 10; //   
			$on_which_user = ceil(db\ratings::get_user_place_param(ID) / $on_page);
			$on_which_user = ($on_which_user < 1) ? 1 : $on_which_user;
			
			$pag = new pagination;
			$pag->page_default = $on_which_user;
			
			
			$limit = $pag->start(db\ratings::stats_count(), 10);
			$sql = "select uc.health + uc.power + uc.agil + uc.intell as score, u.id as id_user, u.login from users_combat uc
			left join users u on u.id = uc.id_user where uc.health + uc.power + uc.agil + uc.intell > 0 order by score desc, u.login asc limit $limit[start], $limit[limit]";
			$sth = db::$pdo->query($sql);
			$rating = $sth->fetchAll(PDO::FETCH_ASSOC);
			return [
				'pag' => $pag,
				'rating' => $rating,
				'on_which_user' => $on_which_user
			];
		}
		
		protected function rating_level_clan_get()
		{
			$pag = new pagination;
			$limit = $pag->start(db\ratings::clan_count(), 10);
			$sql = "select id, name, level as score from clan order by level desc, exp desc limit $limit[start], $limit[limit]";
			$sth = db::$pdo->query($sql);
			$rating = $sth->fetchAll(PDO::FETCH_ASSOC);
			return [
				'pag' => $pag,
				'rating' => $rating
			];
		}
		
		protected function rating_power_clan_get()
		{
			$pag = new pagination;
			$limit = $pag->start(db\ratings::clan_count(), 10);
			$sql = "
			select
				c.id,
				c.name,
				(select
					sum(uc.health) + sum(uc.power) + sum(uc.agil) + sum(uc.intell)
				from users_combat uc
				where id_user in(select cp.id_user from clan_part cp where cp.id_clan = c.id)) as score
			from clan c
			order by score desc
			limit $limit[start], $limit[limit]";
			$sth = db::$pdo->query($sql);
			$rating = $sth->fetchAll(PDO::FETCH_ASSOC);
			foreach ($rating as $key => $val) {
				if (is_null($val['score'])) {
					$rating[$key]['score'] = 0;
				}
			}
			return [
				'pag' => $pag,
				'rating' => $rating
			];
		}
	}
}