<?php

class referral
{
	/* узнать кто пригласил этого игрока (чей он реферал)
	если есть пригласивший, то вернет его id, иначе false */
	public static function who_invited($user_id)
	{
		$sql = 'select id_invited from users where id = ? limit 1';
		$sth = \db::$pdo->prepare($sql);
		$sth->execute([$user_id]);
		$id_invited = $sth->fetchColumn();
		if ($id_invited) return $id_invited;
		return false;
	}
	
	/* получить реферальную ссылку игрока */
	public static function get_link($user_id)
	{
		return subord_url('start', "r=$user_id");
	}
	
	/* какой бонус дает реферал (%) */
	public static function get_bonus()
	{
		return 30;
	}
}

/* бонус от реферала */
class referral_bonus
{
	private
	$user_id, // игрок который покупает мутаген
	$mutagen; // количество мутагена
	
	public function __construct($user_id, $mutagen)
	{
		$this->user_id = $user_id;
		if (is_numeric($mutagen)) {
			$this->mutagen = $mutagen;
		} else {
			$this->mutagen = false;
		}
	}
	
	/* начислить */
	public function overcharging()
	{
		$user_id = $this->user_id;
		$mutagen = $this->mutagen;
		if ( ! $mutagen) {
			return false;
		}
		
		// получить id пригласившего
		$id_invited = referral::who_invited($user_id);
		if ( ! $id_invited) {
			return false;
		}
		
		// рассчитать количество мутагена которое получит пригласивший
		$mutagen_bonus = percentage($mutagen, referral::get_bonus());
		$mutagen_bonus = (int) floor($mutagen_bonus);
		if ($mutagen_bonus < 1) {
			return false;
		}
		
		// начислить
		$currency = new currency($id_invited);
		$currency->bonuses = false;
		$currency->notf = false;
		$currency->base = $mutagen_bonus;
		// в статистику
		$user_statistic = new \user_statistic($id_invited);
		$user_statistic->increase_statistic('mutagen_referral', $mutagen_bonus);
		// в журнал
		$jtitle = 'Система Рефералов';
		$jtext = '%s приобрел <span class="mutagen-color">мутаген</span> в размере <span class="mutagen-color">%s единиц</span>. Вам начислено: <span class="mutagen-color">%s мутагена</span>';
		$user_login = user::login_get($user_id);
		$user_link = subord_url('profile', "id_user=$user_id");
		$jtext = sprintf($jtext, "<a href='$user_link'>$user_login</a>", number_format($mutagen), number_format($mutagen_bonus));
		journal::record_add(['title' => $jtitle, 'text' => $jtext], $id_invited);
	}
}