<?php

namespace indrat
{
	class shop_cards extends \_tbl_line
	{
		const SHOWCASE_CARDS_NUM = 6; // количество (различные типы) карточек на витрине магазина
		const SHOWCASE_UPDATE_DAY = 'Monday'; // в какой день происходит обновление витрины (указывать имя дня в английском формате)
		
		protected static $table = 'shop_cards';
		
		// получить стоимость карточки по id
		public static function card_price_get($card_id)
		{
			$price['mutagen'] = self::get('price_mutagen', 'where id_card = ?', [$card_id]);
			
			$stock = new \stock\stock();
			$discount_cards = $stock->get_discount_cards();
			if ($discount_cards > 0) {
				$price_mutagen_minus = (int) floor(percentage($price['mutagen'], $discount_cards));
				$price['mutagen'] -= $price_mutagen_minus;
			}
			return $price;
		}
		
		// посчитать кол-во карточек
		public static function cards_count()
		{
			return self::get('count(*)');
		}
		
		// добавить карточку на витрину
		public static function card_add($card_id, $price)
		{
			return self::add('id_card = ?, price_mutagen = ?', [$card_id, $price]);
		}
		
		// существует ли карточка на витрине
		public static function card_isset($card_id)
		{
			if (self::get('count(*)', 'where id_card = ?', [$card_id]) > 0) {
				return true;
			} return false;
		}
		
		// получить временную метку отвечающую за обновление витрины
		public static function showcase_timestamp_update_get()
		{
			$sql = 'select showcase_timestamp from shop_cards_detail limit 1';
			$sth = \db::$pdo->query($sql);
			return $sth->fetchColumn();
		}
		
		// перезаписать временную метку отвечающую за обновление витрины
		public static function showcase_timestamp_update_set($time = false)
		{
			$time = ($time) ? $time : time();
			$sql = 'update shop_cards_detail set showcase_timestamp = ? limit 1';
			$sth = \db::$pdo->prepare($sql);
			return $sth->execute([$time]);
		}
		
		// вернет true, если время отвечающее за обновление витрины прошло
		public static function showcase_timestamp_update_gone()
		{
			if (time() >= strtotime('next ' . self::SHOWCASE_UPDATE_DAY, self::showcase_timestamp_update_get())) {
				return true;
			} return false;
		}
		
		// очистить витрину
		public static function showcase_truncate()
		{
			\db::$pdo->exec('truncate shop_cards');
		}
		
	}
}

namespace
{
	class shop_cards extends \indrat\shop_cards
	{
		public static function showcase_ctrl()
		{
			static::showcase_clean_ctrl();
			static::showcase_fill_ctrl();
		}
		
		// контроллер заполнения витрины
		public static function showcase_fill_ctrl()
		{
			$cards_num = self::cards_count();
			if ($cards_num > 0) { // если кол-во карточек на витрине достаточно
				return;
			}
			$cards_id = cards::card_id_norepeat_random(self::SHOWCASE_CARDS_NUM);
			foreach ($cards_id as $id) {
				self::card_add($id, cards::card_price_get($id));
			}
		}
		
		// контроллер очистки витрины
		public static function showcase_clean_ctrl()
		{
			if ( ! self::showcase_timestamp_update_gone()) { // если время еще не прошло
				return;
			}
			self::showcase_truncate(); // очистить витрину
			self::showcase_timestamp_update_set(); // установить новое время
		}
		
		// контроллер покупки карточки
		public static function card_buy_ctrl($card_id, $user_id = false)
		{
			user_id($user_id);
			if ( ! self::card_isset($card_id)) { // если такой карточки на витрине нет
				return;
			}
			$card_price = self::card_price_get($card_id)['mutagen'];
			if (currency::mutagen_get($user_id) < $card_price) { // если средств недостаточно
				return;
			}
			if (user_cards::cards_collected($card_id, $user_id)) { // таких карточек полная коллекция
				return;
			}
			user_cards::card_up_ctrl($card_id, 1, $user_id);
			
			$currency = new currency($user_id);
			$currency->bonuses = false;
			$currency->base = -$card_price;
			$currency->mutagen_add();
		}
	}
	
	class shop_cards_model
	{
		public function shop_cards()
		{
			$user_id = ID;
			$cards = $this->cards_get();
			foreach ($cards as $key => $card) {
				$cards[$key]['name'] = cards::card_name_get($card['id_card']);
				$cards[$key]['price_mutagen'] = shop_cards::card_price_get($card['id_card'])['mutagen'];
				$cards[$key]['param'] = cards::card_param_main_get($card['id_card']);
				$cards[$key]['at_user_num'] = user_cards::card_num_get($card['id_card'], $user_id);
				$cards[$key]['max'] = cards::card_max_get($card['id_card']);
			}
			return [
				'cards' => $cards
			];
		}
		
		private function cards_get()
		{
			$sql = 'select sc.id_card from shop_cards sc order by (%s) desc';
			$sql_inserted = 'select c.health + c.power + c.agil + c.intell from _cards c where c.id = sc.id_card';
			$sql = sprintf($sql, $sql_inserted);
			$sth = db::$pdo->query($sql);
			return $sth->fetchAll(PDO::FETCH_ASSOC);
		}
	}
}