<?php

/* акции */

namespace stock;

class stock
{
	public function __construct() {}
	
	/* id акции, модификатор (скидка, бонус) и срок */
	public function start($id_stock, $modifier, $term)
	{
		$id_stock = (int) $id_stock;
		$modifier = (int) $modifier;
		$term = (int) $term;
		if ( ! $id_stock ||  ! $modifier) {
			return false;
		}
		if ( ! $this->stock_exist($id_stock)) {
			return false;
		}
		
		$starter = new starter($id_stock, $modifier, $term);
		return $starter->start();
	}
	
	public function finish()
	{
		$endinger = new endinger();
		return $endinger->finish();
	}
	
	public function expire()
	{
		$expiration = new expiration();
		return $expiration->expire();
	}
	
	/* проверяеть есть ли активная акция */
	public function is_stock_acting()
	{
		$sql = 'select count(*) from stock_acting limit 1';
		$sth = \db::$pdo->query($sql);
		if ($sth->fetchColumn() > 0) {
			return true;
		} return false;
	}
	
	/* проверяеть есть ли активная акция с таким типом */
	public function is_stock_acting_type($type)
	{
		$sql = 'select count(*) from stock_acting where type = ? limit 1';
		$sth = \db::$pdo->prepare($sql);
		$sth->execute([$type]);
		if ($sth->fetchColumn() > 0) {
			return true;
		} return false;
	}
	
	/* проверяет существует ли акция с таким id */
	public function stock_exist($id)
	{
		$sql = 'select count(*) from _stock where id = ? limit 1';
		$sth = \db::$pdo->prepare($sql);
		$sth->execute([$id]);
		if ($sth->fetchColumn() > 0) {
			return true;
		} return false;
	}
	
	/* получить секунды: сколько осталось до завершения акции */
	public function get_end_of_stock_seconds_left($id_stock = false)
	{
		$stock_acting = new \db\stock_acting();
		$stock = $stock_acting->get($id_stock);
		return ($stock->timestamp + $stock->term) - time();
	}
	
	/* получить скидку на тренировку на шаги */
	public function get_discount_training_steps()
	{
		return $this->get_modifier('training_steps');
	}
	
	/* получить скидку на тренировку на лимит */
	public function get_discount_training_limit()
	{
		return $this->get_modifier('training_limit');
	}
	
	/* получить скидку на ампулы */
	public function get_discount_ampoule()
	{
		return $this->get_modifier('ampoule');
	}
	
	/* получить скидку на улучшение оружия */
	public function get_discount_weapon()
	{
		return $this->get_modifier('weapon');
	}
	
	/* получить скидку на карточки */
	public function get_discount_cards()
	{
		return $this->get_modifier('cards');
	}
	
	/* получить бонус акции к опыту */
	public function get_bonus_expr()
	{
		return $this->get_modifier('expr');
	}
	
	/* получить бонус акции к баксам */
	public function get_bonus_bucks()
	{
		return $this->get_modifier('bucks');
	}
	
	/* получить бонус акции к опыту */
	public function get_bonus_metal()
	{
		return $this->get_modifier('metal');
	}
	
	/* получить скидку на покупку мутагена */
	public function get_discount_mutagen_purchase()
	{
		return $this->get_modifier('mutagen_purchase');
	}
	
	private function get_modifier($type)
	{
		$sql = 'select sum(modifier) from stock_acting where type = ? limit 1';
		$sth = \db::$pdo->prepare($sql);
		$sth->execute([$type]);
		$modifier = $sth->fetchColumn();
		$modifier = ($modifier) ? $modifier : 0;
		return $modifier;
	}
}

/* класс запуска акции */
class starter
{
	public
		$id_stock,
		$modifier,
		$term;
	
	public function __construct($id_stock, $modifier, $term)
	{
		$this->id_stock = $id_stock;
		$this->modifier = $modifier;
		$this->term = $term;
	}
	
	public function start()
	{
		$stock = new stock();
		$is_stock = $stock->is_stock_acting();
		if ($is_stock) {
			$stock_acting = new \db\stock_acting();
			$stock_acting = $stock_acting->get();
			$term = $stock_acting->term;
			$timestamp = $stock_acting->timestamp;
		} else {
			$term = $this->term;
			$timestamp = time();
		}
		
		$stock_detail = $this->get_stock_detail();
		$stock_acting = new \db\stock_acting();
		
		$is_stock_type = $stock->is_stock_acting_type($stock_detail->type);
		if ($is_stock_type) {
			$id = $stock_acting->get_id_by_type($stock_detail->type);
			$stock_acting->update($id, 'modifier', $this->modifier);
		} else {
			return $stock_acting->insert([
				'id_stock' => $this->id_stock,
				'type' => $stock_detail->type,
				'text' => $stock_detail->text,
				'link' => $stock_detail->link,
				'modifier' => $this->modifier,
				'plus' => $stock_detail->plus,
				'term' => $term,
				'timestamp' => $timestamp
			]);
		}
	}
	
	private function get_stock_detail()
	{
		return (new \db\stock($this->id_stock))->get();
	}
}

/* окончатель акции */
class endinger
{
	public function finish()
	{
		$sql = 'delete from stock_acting';
		\db::$pdo->exec($sql);
	}
}

/* истечение акции */
class expiration
{
	public function expire()
	{
		$stock = new stock();
		$is_stock = $stock->is_stock_acting();
		if ( ! $is_stock) {
			return false;
		}
		$seconds_left = $stock->get_end_of_stock_seconds_left();
		if ($seconds_left > 0) {
			return false;
		}
		$endinger = new endinger();
		$endinger->finish();
		return true;
	}
}

/* коллекция акций */
class stock_collect
{
	public function get_all()
	{
		$sql = 'select * from _stock order by id asc';
		$sth = \db::$pdo->query($sql);
		return $sth->fetchAll(\PDO::FETCH_OBJ);
	}
}