<?php

namespace db;

class stock_acting
{
	private $table = 'stock_acting';
	
	private $last_id;
	
	public function __construct()
	{
		
	}
	
	/* (array) обязательны для установки такие параметры как: id_stock, type, text, modifier, term */
	public function insert($fields)
	{
		$fields = (object) [
			'id_stock' => $fields['id_stock'],
			'type' => $fields['type'],
			'text' => $fields['text'],
			'link' => (array_key_exists('link', $fields)) ? $fields['link'] : null,
			'modifier' => $fields['modifier'],
			'plus' => (array_key_exists('plus', $fields)) ? $fields['plus'] : 1,
			'term' => $fields['term'],
			'timestamp' => (array_key_exists('timestamp', $fields)) ? $fields['timestamp'] : time()
		];
		
		$sql = "insert into $this->table set id_stock = :id_stock, type = :type, text = :text, link = :link, modifier = :modifier, plus = :plus, term = :term, timestamp = :timestamp";
		$sth = \db::$pdo->prepare($sql);
		$sth->bindParam(':id_stock', $fields->id_stock);
		$sth->bindParam(':type', $fields->type);
		$sth->bindParam(':text', $fields->text);
		$sth->bindParam(':link', $fields->link);
		$sth->bindParam(':modifier', $fields->modifier);
		$sth->bindParam(':plus', $fields->plus);
		$sth->bindParam(':term', $fields->term);
		$sth->bindParam(':timestamp', $fields->timestamp);
		$sth->execute();
		return $this->last_id = \db::$pdo->lastInsertId();
	}
	
	public function update($id, $field_name, $field_value)
	{	
		$sql = "update $this->table set $field_name = :$field_name where id = :id limit 1";
		$sth = \db::$pdo->prepare($sql);
		$sth->bindParam(":$field_name", $field_value);
		$sth->bindParam(":id", $id);
		return $sth->execute();
	}
	
	/* получить все акции */
	public function get_all()
	{
		$sql = "select * from $this->table";
		$sth = \db::$pdo->query($sql);
		return $sth->fetchAll(\PDO::FETCH_OBJ);
	}
	
	/* получить одну акицию (если не указан параметр $id_stock, то возвращает последнюю) */
	public function get($id_stock = false)
	{
		$sql = "select * from $this->table %s limit 1";
		if ($id_stock) {
			$sql = sprintf($sql, 'where id = ?');
			$sth = \db::$pdo->prepare($sql);
			$sth->execute([$id_stock]);
		} else {
			$sql = sprintf($sql, 'order by id desc');
			$sth = \db::$pdo->prepare($sql);
			$sth->execute();
		}
		return $sth->fetch(\PDO::FETCH_OBJ);
	} 
	
	/* получить id рядка по типу */
	public function get_id_by_type($type)
	{
		$sql = "select id from $this->table where type = ? limit 1";
		$sth = \db::$pdo->prepare($sql);
		$sth->execute([$type]);
		return $sth->fetchColumn();
	}
}