<?php

interface interface_support
{
	/*    */
	const APP_LENGTH = 500;
	
	/*    */
	const APP_COMMENT_LENGTH = 300;
	
	/*   */
	public function create_app($app_text);
	
	/*   */
	public function close_app();
	
	/*   */
	public function open_app();
	
	/*   */
	public function add_comment($comment_text);
	
	/*  .  */
	public function clean();
}

class support implements interface_support
{
	public $user_id;
	public $app_id;
	
	public function __construct($user_id = false)
	{
		if ($user_id) {
			$this->user_id = $user_id;
		} else {
			$this->user_id = ID;
		}
	}
	
	public function create_app($app_text)
	{
		if ( ! support_filter::app_text($app_text)) { //     
			return false;
		}
		$sql = 'insert into support set id_user = ?, text = ?, status = ?, timestamp = ?';
		$sth = db::$pdo->prepare($sql);
		$result = $sth->execute([$this->user_id, $app_text, 1, time()]);
		if ($result) {
			$last_id = db::$pdo->lastInsertID();
			$sql = 'insert into support_notice set id_app = ?, for_admin = ?';
			$sth = db::$pdo->prepare($sql);
			$sth->execute([$last_id, 1]);
			return $last_id;
		} return false;
	}
	
	public function close_app()
	{
		$user_position = user::position_get($this->user_id);
		if ($user_position < 2) { //      
			return false;
		}
		$app = $this->get_app();
		if ($app->status == 2) { //     
			return false;
		}
		$sql = 'update support set status = 2 where id = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		return $sth->execute([$this->app_id]);
	}
	
	public function open_app()
	{
		$user_position = user::position_get($this->user_id);
		if ($user_position < 2) { //      
			return false;
		}
		$app = $this->get_app();
		if ($app->status == 1) { //     
			return false;
		}
		$sql = 'update support set status = 1 where id = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		return $sth->execute([$this->app_id]);
	}
	
	public function add_comment($comment_text)
	{
		if ( ! support_filter::app_comment_text($comment_text)) { //     
			return false;
		}
		$app = $this->get_app();
		$user_position = user::position_get($this->user_id);
		if ($app->status == 2 && $user_position < 2) { //           (        )
			return false;
		}
		$sql = 'insert into support_comments set id_app = ?, id_user = ?, text = ?, timestamp = ?';
		$sth = db::$pdo->prepare($sql);
		$result = $sth->execute([$this->app_id, $this->user_id, $comment_text, time()]);
		if ($result) {
			if ($user_position > 1) { //      
				$this->increase_counter_for_user();
			} else { //     
				$this->increase_counter_for_admin();
			}
			notf::set_notf(' ');
			return true;
		} else return false;
	}
	
	private function increase_counter_for_user($n = 1)
	{
		$sql = 'update support_notice set for_user = for_user + ? where id_app = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		return $sth->execute([$n, $this->app_id]);
	}
	
	private function increase_counter_for_admin($n = 1)
	{
		$sql = 'update support_notice set for_admin = for_admin + ? where id_app = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		return $sth->execute([$n, $this->app_id]);
	}
	
	public function reset_counter_for_user()
	{
		$sql = 'update support_notice set for_user = 0 where id_app = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		return $sth->execute([$this->app_id]);
	}
	
	public function reset_counter_for_admin()
	{
		$sql = 'update support_notice set for_admin = 0 where id_app = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		return $sth->execute([$this->app_id]);
	}
	
	public function clean()
	{
		$user_position = user::position_get($this->user_id);
		if ($user_position < 2) { //     
			return false;
		}
		$sql = 'truncate table %s';
		$app_num = db::$pdo->exec(sprintf($sql, 'support'));
		$comments_num = db::$pdo->exec(sprintf($sql, 'support_comments'));
		$notice_num = db::$pdo->exec(sprintf($sql, 'support_notice'));
		$notf = " $app_num   $comments_num . {-$app_num + $comments_num + $notice_num rows}";
		notf::set_notf($notf, 'default');
		return true;
	}
	
	/*    */
	public function get_app()
	{
		$sql = 'select * from support where id = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$this->app_id]);
		return $sth->fetch(PDO::FETCH_OBJ);
	}
	
	/*  n   */
	public function get_app_all($n = 10)
	{
		$app_num = $this->count_app();
		$pag = new pagination;
		$limit = $pag->start($app_num, $n);
		$sql = "select s.*, sn.for_user, sn.for_admin from support s ";
		$sql .= "left join support_notice sn on sn.id_app = s.id ";
		$sql .= "order by s.status asc, s.id desc limit $limit[start], $limit[limit]";
		$sth = db::$pdo->prepare($sql);
		$sth->execute();
		return [
			'pag' => $pag,
			'app' => $sth->fetchAll(PDO::FETCH_OBJ)
		];
	}
	
	/*  n    */
	public function get_user_app_all($n = 10)
	{
		$app_num = $this->count_app();
		$pag = new pagination;
		$limit = $pag->start($app_num, $n);
		$sql = "select s.*, sn.for_user, sn.for_admin from support s ";
		$sql .= "left join support_notice sn on sn.id_app = s.id ";
		$sql .= "where s.id_user = ? order by s.status asc, s.id desc limit $limit[start], $limit[limit]";
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$this->user_id]);
		return [
			'pag' => $pag,
			'app' => $sth->fetchAll(PDO::FETCH_OBJ)
		];
	}
	
	/*  n    */
	public function get_last_app_comments($n = 10)
	{
		$app_comments_num = $this->count_app_comments();
		$pag = new pagination;
		$limit = $pag->start($app_comments_num, $n);
		$sql = "select * from support_comments where id_app = ? order by id desc limit $limit[start], $limit[limit]";
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$this->app_id]);
		return [
			'pag' => $pag,
			'app_comments' => $sth->fetchAll(PDO::FETCH_OBJ)
		];
	}
	
	/*     */
	public function count_app()
	{
		$sql = 'select count(*) from support where id_user = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$this->user_id]);
		return $sth->fetchColumn();
	}
	
	/*     */
	public function count_app_comments()
	{
		$sql = 'select count(*) from support_comments where id_app = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$this->app_id]);
		return $sth->fetchColumn();
	}
	
	/*      id */
	public function isset_app()
	{
		$sql = 'select count(*) from support where id = ? limit 1';
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$this->app_id]);
		if ($sth->fetchColumn() > 0) {
			return true;
		} return false;
	}
	
	/*      .  */
	public function get_new_comments()
	{
		$user_position = user::position_get($this->user_id);
		if ($user_position < 2) {
			$sql = 'select sum(for_user) from support_notice where id_app in(select id from support where id_user = ?)';
			$sth = db::$pdo->prepare($sql);
			$sth->execute([$this->user_id]);
		} else {
			$sql = "select sum(for_admin) from support_notice limit 1";
			$sth = db::$pdo->query($sql);
		}
		return $sth->fetchColumn();
	}
}

class support_filter
{
	/*    */
	public static function app_id($id)
	{
		if ( ! is_numeric($id)) {
			return false;
		}
		if ($id < 1) {
			return false;
		}
		$support = new support();
		$support->app_id = $id;
		if ( ! $support->isset_app()) {
			return false;
		} return true;
	}
	
	/*    */
	public static function app_text($text)
	{
		if (is_null($text)) {
			return false;
		}
		$len = mb_strlen($text);
		if ($len < 1) {
			return false;
		}
		if ($len > support::APP_LENGTH) {
			return false;
		}
		return htmlspecialchars($text);
	}
	
	/*    */
	public static function app_comment_text($text)
	{
		if (is_null($text)) {
			return false;
		}
		$len = mb_strlen($text);
		if ($len < 1) {
			return false;
		}
		if ($len > support::APP_COMMENT_LENGTH) {
			return false;
		}
		return htmlspecialchars($text);
	}
}