<?php

class training
{
	public static function get_training_param_increase()
	{
		$config = require ROOT . 'project/internal/config/main.config.php';
		return $config['training_param_increase'];
	}
	
	public static function get_training_limit_param_increase()
	{
		$config = require ROOT . 'project/internal/config/main.config.php';
		return $config['training_limit_param_increase'];
	}
	
	// узнать количество пройденых тренировок за всё время по типу
	public static function get_training_completed_by_id($id_user, $type)
	{
		$field = "training_{$type}_completed";
		$sql = 'select %s from users_training where id_user = ? limit 1';
		$sql = sprintf($sql, $field);
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_user]);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// узнать реально возможное количество тренировок в зависимости от уровня лимита
	public static function get_training_limit_by_limit_level($limit_level)
	{
		$sql = 'select scope from _training_limits where level = ? limit 1';
		$sth = \db::$pdo->prepare($sql);
		$sth->execute([$limit_level]);
		return $sth->fetchColumn();
	}
	
	// узнать стоимость тренировки шага
	public static function get_training_price_bucks($step)
	{
		$sql = 'select price_bucks from _training_steps where step = ? limit 1';
		$sth = \db::$pdo->prepare($sql);
		$sth->execute([$step]);
		$price_bucks = $sth->fetchColumn();
		
		$stock = new stock\stock();
		$discount_training_steps = $stock->get_discount_training_steps();
		if ($discount_training_steps > 0) {
			$price_bucks_minus = (int) floor(percentage($price_bucks, $discount_training_steps));
			$price_bucks = $price_bucks - $price_bucks_minus;
		}
		return $price_bucks;
	}
	
	// узнать уровень лимита определенной тренировки игрока
	// type: health, power, agil, intell
	public static function get_training_limit_level($id_user, $type)
	{
		$field = "training_{$type}_limit_level";
		$sql = 'select %s from users_training where id_user = ? limit 1';
		$sql = sprintf($sql, $field);
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$id_user]);
		return $sth->fetch(PDO::FETCH_NUM)[0];
	}
	
	// узнать максимальный уровень лимита
	public static function get_training_limit_level_max()
	{
		$sql = 'select max(level) from _training_limits limit 1';
		$sth = \db::$pdo->query($sql);
		return $sth->fetchColumn();
	}
	
	// узнать максимальный лимит
	public static function get_training_limit_max()
	{
		$sql = 'select max(scope) from _training_limits limit 1';
		$sth = \db::$pdo->query($sql);
		return $sth->fetchColumn();
	}
	
	// получить массив с ценами на уровень лимита
	public static function get_training_limit_level_price($limit_level)
	{
		$sql = 'select price_mutagen from _training_limits where level = ? limit 1';
		$sth = \db::$pdo->prepare($sql);
		$sth->execute([$limit_level]);
		$price['mutagen'] = $sth->fetchColumn();
		$price['pancers'] = 0;
		
		$stock = new stock\stock();
		$discount_training_limit = $stock->get_discount_training_limit();
		if ($discount_training_limit > 0) {
			$price_mutagen_minus = (int) floor(percentage($price['mutagen'], $discount_training_limit));
			$price['mutagen'] -= $price_mutagen_minus;
		}
		return $price;
	}
	
	// изменить уровень лимита определенного типа тренировки
	public static function training_limit_level_update($id_user, $type, $num = 1)
	{
		$field = "training_{$type}_limit_level";
		$sql = "update users_training set $field = $field + ? where id_user = ? limit 1";
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$num, $id_user]);
	}
	
	// изменить значение счетчика completed
	public static function training_completed_update($id_user, $type, $num = 1)
	{
		$field = "training_{$type}_completed";
		$sql = "update users_training set $field = $field + ? where id_user = ? limit 1";
		$sth = db::$pdo->prepare($sql);
		$sth->execute([$num, $id_user]);
	}
	
	// контроллер увеличения уровня лимита
	public static function limit_level_up_ctrl($type, $user_id = false)
	{
		user_id($user_id);
		if (self::get_training_limit_level($user_id, $type) >= self::get_training_limit_level_max($type)) { // если уровень уже максимальный
			return false;
		}
		$mutagen = user_currency::get_mutagen($user_id);
		$pancers = user_currency::get_pancers($user_id);
		$limit_level = self::get_training_limit_level($user_id, $type);
		$price = training::get_training_limit_level_price($limit_level, $type);
		if ($mutagen < $price['mutagen'] || $pancers < $price['pancers']) { // средств не хватает
			\notf::set_notf('Недостаточно средств', 'error');
			return false;
		}
		training::training_limit_level_update($user_id, $type);
		uc::update_param($user_id, $type, self::get_training_limit_param_increase());
		$currency = new \currency($user_id);
		$currency->bonuses = false;
		$currency->base = -$price['mutagen'];
		$currency->mutagen_add();
		
		$trans = [
			'health' => 'выносливость',
			'power' => 'силу',
			'agil' => 'ловкость',
			'intell' => 'интеллект'
		];
		$notf = "Лимит тренировок на $trans[$type] увеличен.";
		\notf::set_notf($notf, 'default');
	}
	
	// контроллер тренировки
	public static function training_ctrl($type, $user_id = false)
	{
		user_id($user_id);
		$limit_level = training::get_training_limit_level($user_id, $type);
		$limit = training::get_training_limit_by_limit_level($limit_level, $type);
		$training_completed = training::get_training_completed_by_id($user_id, $type);
		if ($training_completed >= $limit) { // если лимит превышен
			return;
		}
		$training_price_bucks = self::get_training_price_bucks($training_completed);
		$user_bucks = user_currency::get_bucks($user_id);
		if ($user_bucks < $training_price_bucks) {
			\notf::set_notf('Недостаточно средств', 'error');
			return;
		}
		
		training::training_completed_update($user_id, $type);
		uc::update_param($user_id, $type, self::get_training_param_increase());
		
		$currency = new \currency($user_id);
		$currency->bonuses = false;
		$currency->base = -$training_price_bucks;
		$currency->bucks_add();
	}
}

class training_model
{
	public function get()
	{
		$training_completed = [
			'health' => training::get_training_completed_by_id(ID, 'health'),
			'power' => training::get_training_completed_by_id(ID, 'power'),
			'agil' => training::get_training_completed_by_id(ID, 'agil'),
			'intell' => training::get_training_completed_by_id(ID, 'intell')
		];
		
		$training_price_bucks = [
			'health' => training::get_training_price_bucks($training_completed['health']),
			'power' => training::get_training_price_bucks($training_completed['power']),
			'agil' => training::get_training_price_bucks($training_completed['agil']),
			'intell' => training::get_training_price_bucks($training_completed['intell'])
		];
		
		
		
		$training_limit_level = [
			'health' => training::get_training_limit_level(ID, 'health'),
			'power' => training::get_training_limit_level(ID, 'power'),
			'agil' => training::get_training_limit_level(ID, 'agil'),
			'intell' => training::get_training_limit_level(ID, 'intell')
		];
		$training_limit = [
			'health' => training::get_training_limit_by_limit_level($training_limit_level['health']),
			'power' => training::get_training_limit_by_limit_level($training_limit_level['power']),
			'agil' => training::get_training_limit_by_limit_level($training_limit_level['agil']),
			'intell' => training::get_training_limit_by_limit_level($training_limit_level['intell'])
		];
		
		$training_limit_level_max = training::get_training_limit_level_max();
		$training_limit_level_max = [
			'health' => $training_limit_level_max,
			'power' => $training_limit_level_max,
			'agil' => $training_limit_level_max,
			'intell' => $training_limit_level_max
		];
		
		$training_price = [
			'health' => training::get_training_limit_level_price($training_limit_level['health']),
			'power' => training::get_training_limit_level_price($training_limit_level['power']),
			'agil' => training::get_training_limit_level_price($training_limit_level['agil']),
			'intell' => training::get_training_limit_level_price($training_limit_level['intell'])
		];
		
		return [
			'training_completed' => $training_completed,
			'training_limit' => $training_limit,
			'training_price' => $training_price,
			'training_limit_level' => $training_limit_level,
			'training_limit_level_max' => $training_limit_level_max,
			'training_price_bucks' => $training_price_bucks
		];
	}
}