<?php

namespace indrat
{
	class weapon
	{
		const TABLE_LVLS = '_weapon';
		
		// получить название оружия по черепашке (leo, raf...)
		public static function name_get($turtle)
		{
			$names = [
				'leo' => 'Катаны',
				'raf' => 'Кинжалы-сай',
				'doni' => 'Шест-бо',
				'miki' => 'Нунчаки '
			];
			return $names[$turtle];
		}
		
		public static function bonus_param_get($level)
		{
			$sql = 'select bonus_param from ' . self::TABLE_LVLS . ' where level = ? limit 1';
			$sth = \db::$pdo->prepare($sql);
			$sth->execute([$level]);
			$bonus = $sth->fetchColumn();
			return [
				'health' => $bonus,
				'power' => $bonus,
				'agil' => $bonus,
				'intell' => $bonus
			];
		}
		
		// level_next - насколько уровней вперед продвинуться относительно level_current
		public static function bonus_param_next_get($level_current, $level_next)
		{
			$bonus_param = self::bonus_param_get($level_current);
			$af = function ($a, $b) {
				return $a - $b;
			};
			$bonus_param_next = weapon::bonus_param_get($level_current + $level_next);
			$bonus_param_next = array_map($af, $bonus_param_next, $bonus_param);
			$bonus_param_next['health'] = $bonus_param_next[0];
			$bonus_param_next['power'] = $bonus_param_next[1];
			$bonus_param_next['agil'] = $bonus_param_next[2];
			$bonus_param_next['intell'] = $bonus_param_next[3];
			return $bonus_param_next;
		}
		
		public static function bonus_victory_chance_percent_get($level)
		{
			return \package('turtles-weapon')[$level]['bonus_victory_chance_percent'];
		}
		
		public static function bonus_fighting_score_get($level)
		{
			$sql = 'select bonus_fighting_score from ' . self::TABLE_LVLS . ' where level = ? limit 1';
			$sth = \db::$pdo->prepare($sql);
			$sth->execute([$level]);
			return $sth->fetchColumn();
		}
		
		public static function bonus_scramble_combat_expr_get($level)
		{
			$sql = 'select bonus_combat_expr from ' . self::TABLE_LVLS . ' where level = ? limit 1';
			$sth = \db::$pdo->prepare($sql);
			$sth->execute([$level]);
			return $sth->fetchColumn();
		}
		
		public static function price_improv_get($level)
		{
			$sql = 'select price_mutagen from ' . self::TABLE_LVLS . ' where level = ? limit 1';
			$sth = \db::$pdo->prepare($sql);
			$sth->execute([$level]);
			$price_mutagen = $sth->fetchColumn();
			
			$stock = new \stock\stock();
			$discount_weapon = $stock->get_discount_weapon();
			if ($discount_weapon > 0) {
				$price_mutagen_minus = (int) floor(percentage($price_mutagen, $discount_weapon));
				$price_mutagen -= $price_mutagen_minus;
			}
			return $price_mutagen;
		}
		
		public static function level_min_get()
		{
			$sql = 'select min(level) from ' . self::TABLE_LVLS . ' limit 1';
			$sth = \db::$pdo->prepare($sql);
			$sth->execute();
			return $sth->fetchColumn();
		}
		
		public static function level_max_get()
		{
			$sql = 'select max(level) from ' . self::TABLE_LVLS . ' limit 1';
			$sth = \db::$pdo->prepare($sql);
			$sth->execute();
			return $sth->fetchColumn();
		}
	}
	
	class user_weapon extends \_tbl_line
	{
		protected static $table = 'users_weapon';
		
		public static function level_get($user_id = false)
		{
			return self::get('level', 'where id_user = ?', [$user_id]);
		}
		
		public static function level_set($level, $user_id = false)
		{
			return self::set('level = ?', 'where id_user = ?', [$level, $user_id]);
		}
		
		public static function level_up($user_id = false)
		{
			return self::set('level = level + 1', 'where id_user = ?', [$user_id]);
		}
	}
}

namespace
{
	class weapon extends \indrat\weapon
	{
		
	}
	
	class user_weapon extends \indrat\user_weapon
	{
		public static function weapon_level_up_ctrl()
		{
			$user_id = ID;
			$weapon_level = user_weapon::level_get($user_id);
			if ($weapon_level >= weapon::level_max_get()) { // если уровень максимальный
				return;
			}
			$price_improv = weapon::price_improv_get($weapon_level);
			if (user_currency::get_mutagen($user_id) < $price_improv) { // если средств недостаточно
				return;
			}
			$bonus_param = weapon::bonus_param_next_get($weapon_level, 1);
			user_currency_indrat::update_mutagen(-$price_improv, $user_id); // снять мутаген
			user_combat::update_param_all($user_id, $bonus_param); // увеличить параметры
			self::level_up($user_id);
		}
	}
	
	class weapon_model
	{
		public function weapon()
		{
			$user_id = ID;
			$turtle = user::turtle_get($user_id);
			$turtle_name = translate::turtle_name($turtle);
			$weapon_level = user_weapon::level_get($user_id);
			$weapon_level_max = weapon::level_max_get();
			$weapon_name = weapon::name_get($turtle);
			
			$param_name = translate::param_name();
			$bonus_param = weapon::bonus_param_get($weapon_level);
			$bonus_fighting_score = weapon::bonus_fighting_score_get($weapon_level);
			$bonus_scramble_combat_expr = weapon::bonus_scramble_combat_expr_get($weapon_level);
			$price_improv = weapon::price_improv_get($weapon_level);
			
			$bonus_param_next = ($weapon_level < $weapon_level_max) ? weapon::bonus_param_next_get($weapon_level, 1) : false;
			$bonus_fighting_score_next = ($weapon_level < $weapon_level_max) ? weapon::bonus_fighting_score_get($weapon_level + 1) - $bonus_fighting_score : false;
			$bonus_scramble_combat_expr_next = ($weapon_level < $weapon_level_max) ? weapon::bonus_scramble_combat_expr_get($weapon_level + 1) - $bonus_scramble_combat_expr : false;
			
			return [
				'turtle' => $turtle,
				'turtle_name' => $turtle_name,
				'weapon_name' => $weapon_name,
				'weapon_level' => $weapon_level,
				'weapon_level_max' => $weapon_level_max,
				'param_name' => $param_name,
				'bonus_param' => $bonus_param,
				'bonus_fighting_score' => $bonus_fighting_score,
				'bonus_scramble_combat_expr' => $bonus_scramble_combat_expr,
				'bonus_param_next' => $bonus_param_next,
				'bonus_fighting_score_next' => $bonus_fighting_score_next,
				'bonus_scramble_combat_expr_next' => $bonus_scramble_combat_expr_next,
				'price_improv' => $price_improv
			];
		}
	}
}