<?php

function at_minutes($seconds)
{
	return intval($seconds / 60);
}

function at_hours($seconds)
{
	return intval($seconds / 3600);
}

function at_days($seconds)
{
	return intval($seconds / 86400);
}

function at_months($seconds)
{
	return intval($seconds / (86400 * 30));
}

function at_years($seconds)
{
	return intval($seconds / (86400 * 365));
}

/* function seconds_format($seconds)
{
	if ($seconds <= 1) {
		return 'Только что';
	}
	if ($seconds < 60) {
		return "$seconds секунд";
	}
	if ($seconds >= 60 && $seconds <= 86400) {
		return 'Недавно';
	}
	if ($seconds >= 86400 && $seconds < 172800) {
		return 'Вчера';
	}
	else {
		return 'Давно';
	}
} */

/**
 * конвертация секунд в корректное для воспроятия человека время (т.е. минуты, часы, дни и т.д.)
 */
function conversec($seconds)
{
	$time = [
		'years'			=> 0,
		'months'			=> 0,
		'days'			=> 0,
		'hours'			=> 0,
		'minutes'		=> 0,
		'seconds'		=> intval($seconds % 60),
		'micros'			=> 0
	];
	
	if ($seconds < 60) {
		return $time;
	}
	$time['minutes'] = intval($seconds % 3600 / 60);
	if ($seconds / 60 < 60) {
		return $time;
	}
	$time['hours'] = intval($seconds % 86400 / 3600);
	if ($seconds / 3600 < 24) {
		return $time;
	}
	$time['days'] = intval($seconds % (86400 * 30) / 86400);
	if ($seconds / 86400 < 30) {
		return $time;
	}
	$time['months'] = intval($seconds % (86400 * 360) / (86400 * 30));
	if ($seconds / (86400 * 30) < 12) {
		return $time;
	}
	$time['years'] = intval($seconds / (86400 * 360));
	return $time;
}

// Принимает время в секундах
// И возвращает отформатированную строку
/**
 * параметр $edit принимает массив из двух других массивов
 * в первом из них (обязательно с ключем 0) указываются единицы времени которые должны обратаватся (в итоге войти в конечную строку), все остальные единицы времени будут удалены
 * во втором (обязательно с ключем 1) указываются единицы времени которые должны быть удалены (все остальные остаются)
 * примеры:
 * [0 => ['seconds', 'minutes']] // на выходе получаем только секунды и минуты
 * [1 => ['years', 'months']] // на выходе: всё кроме годов и месяцев
 * примечание: нежелательно удалять меньшие единицы времени ибо если секунд для преобразования будет недостаточно для больших - вернется пустая строка
 */
function format_time($seconds, $edit = false, $trans = false)
{
	$time	= conversec($seconds);
	if (isset($edit[0])) {
		foreach ($time as $key => $val) {
			if (in_array($key, $edit[0]) === false) {
				unset($time[$key]);
			}
		}
	}
	if (isset($edit[1])) {
		foreach ($time as $key => $val) {
			if (in_array($key, $edit[1]) === true) {
				unset($time[$key]);
			}
		}
	}
	$frmt = [
		'micros' => 'мс. ',
		'seconds' => 'с. ',
		'minutes' => 'м. ',
		'hours' => 'ч. ',
		'days' => 'д. ',
		'months' => 'мес. ',
		'years' => 'г. '
	];
	
	$str = '';
	foreach ($time as $key => $unit) {
		if ($unit > 0) {
			$str .= "$unit$frmt[$key]";
		}
	}
	
	return $str;
}

// Наступил ли $n-ый день?
// Защитывается только после полуночи
function day_new($unix, $n = 0)
{
	if ($n == 1) $n = 0;
	if ($n > 0) {
		$unix = strtotime("00:00:00 +$n day", $unix);
	}
	
	$date = getdate();
	$last = getdate($unix);
	
	if ($last['year'] < $date['year']) {
		return true;
	}
	else {
		if ($last['mon'] < $date['mon']) {
			return true;
		}
		else {
			if ($last['mday'] < $date['mday']) {
				return true;
			}
			else {
				return false;
			}
		}
	}
}

// наступил ли новый час
// функция имеет такое же поведение как и day_new, то есть считает не целый час (3600 сек.), а только единицы часов в этих же пределах
function hour_new($unix, $n = 0)
{
	if ($n > 0) {
		$unix = strtotime("+$n hour", $unix);
	}
	
	$date = getdate();
	$last = getdate($unix);
	
	if ($last['year'] < $date['year']) {
		return true;
	}
	else {
		if ($last['mon'] < $date['mon']) {
			return true;
		}
		else {
			if ($last['mday'] < $date['mday']) {
				return true;
			}
			else {
				if ($last['hours'] < $date['hours']) {
					return true;
				}
			}
		}
	}
	return false;
}

// прошел ли заданный промежуток времени (сек.) относительно метки времени передаваемой первым параметром
function utime_new($unix, $gap)
{
	if ((time() - $unix) > $gap) {
		return true;
	}
	return false;
}

/**
 * прошедшие дни (защитывается только после полуночи, т.е. день защитается даже если в функцию будет послано время наступившее 10 секунд до полуночи)
 */
function last_days($unix)
{
	$date = getdate();
	$last = getdate($unix);
	if ($last['year'] <= $date['year']) {
		$factor = $date['year'] - $last['year'];
		$days_last = $date['yday'] - $last['yday'];
		if ($factor > 0) {
			$days_last += (364 * $factor);
		} return $days_last;
	}
	return false;
}

/**
 * замеряет время между двумя точками
 * принимает время возвращаемое функцией microtime(true) и предел округления результата
 */
function measure_microtime($start, $double = 3)
{
	$finish	= microtime(true);
	$result	= round($finish - $start, $double);
	return $result;
}

// получить количество секунд оставшихся до полуночи $n-го дня
function left_seconds_midnight($n = 1)
{
	return strtotime("00:00:00 +$n day") - time();
}

// наступило ли $back минут до круглого часа
// прим. если back 10: 10:50 вернет true, 11:00 >=, уже false и т.д.
function back_of($back)
{
	$t0 = time();
	for ($i = 0; $i <= 23; $i++) {
		$t = strtotime("today $i:00 -$back min");
		if ($i == 23) {
			$t2 = strtotime("tomorrow $i:00");
		} else {
			$t2 = strtotime("today $i:00");
		}
		$b = false;
		if ($t0 >= $t && $t0 < $t2) {
			$b = true;
			break;
		}
	}
	return $b;
}

// не прошло ли еще $front минут после круглого часа
function front_of($front)
{
	$t0 = time();
	for ($i = 0; $i <= 23; $i++) {
		$t = strtotime("today $i:00");
		$t2 = strtotime("today $i:00 +$front min");
		
		$b = false;
		if ($t0 >= $t && $t0 < $t2) {
			$b = true;
			break;
		}
	}
	return $b;
}